@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2012-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,13 +16,17 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
  */
 #include "stress-ng.h"
+#include "core-capabilities.h"
+
+#if defined(HAVE_SYS_FANOTIFY_H)
+#include <sys/fanotify.h>
+#endif
+
+#if defined(HAVE_SYS_SELECT_H)
+#include <sys/select.h>
+#endif
 
 static const stress_help_t help[] = {
 	{ NULL,	"fanotify N",	  "start N workers exercising fanotify events" },
@@ -40,11 +45,12 @@ static const stress_help_t help[] = {
 
 /* fanotify stats */
 typedef struct {
-	uint64_t	open;
-	uint64_t	close_write;
-	uint64_t	close_nowrite;
-	uint64_t	access;
-	uint64_t	modify;
+	uint64_t open;		/* # opens */
+	uint64_t close_write;	/* # close writes */
+	uint64_t close_nowrite;	/* # close no writes */
+	uint64_t access;	/* # accesses */
+	uint64_t modify;	/* # modifications */
+	uint64_t rename;	/* # renames */
 } stress_fanotify_account_t;
 
 #endif
@@ -118,6 +124,12 @@ static const unsigned int fan_stress_settings[] = {
 #endif
 #if defined(FAN_Q_OVERFLOW)
 	FAN_Q_OVERFLOW,
+#endif
+#if defined(FAN_RENAME)
+	FAN_RENAME,
+#endif
+#if defined(FAN_FS_ERROR)
+	FAN_FS_ERROR,
 #endif
 	0
 };
@@ -154,6 +166,9 @@ static const unsigned int init_flags[] =
 #if defined(FAN_REPORT_PIDFD)
 	FAN_REPORT_PIDFD,
 #endif
+#if defined(FAN_REPORT_TARGET_FID)
+	FAN_REPORT_TARGET_FID,
+#endif
 };
 
 static char *mnts[MAX_MNTS];
@@ -248,7 +263,7 @@ static void fanotify_event_init_invalid(void)
  */
 static int test_fanotify_mark(const char *name, char *mounts[])
 {
-	int ret_fd, ret;
+	int ret_fd;
 #if defined(FAN_MARK_INODE)
 	const int bad_fd = stress_get_bad_fd();
 #endif
@@ -257,54 +272,51 @@ static int test_fanotify_mark(const char *name, char *mounts[])
 	if (ret_fd < 0) {
 		pr_err("%s: cannot initialize fanotify, errno=%d (%s)\n",
 			name, errno, strerror(errno));
-		return -1;
+		return -errno;
 	}
 
 	/* Exercise fanotify_mark with invalid mask */
 #if defined(FAN_MARK_MOUNT)
-	ret = fanotify_mark(ret_fd, FAN_MARK_ADD | FAN_MARK_MOUNT,
-			~0U, AT_FDCWD, mounts[0]);
-	(void)ret;
+	VOID_RET(int, fanotify_mark(ret_fd, FAN_MARK_ADD | FAN_MARK_MOUNT,
+			~0U, AT_FDCWD, mounts[0]));
 #endif
 
 	/* Exercise fanotify_mark with invalid flag */
-	ret = fanotify_mark(ret_fd, ~0U, FAN_ACCESS, AT_FDCWD, mounts[0]);
-	(void)ret;
+	VOID_RET(int, fanotify_mark(ret_fd, ~0U, FAN_ACCESS, AT_FDCWD, mounts[0]));
 
 	/* Exercise fanotify_mark on bad fd */
 #if defined(FAN_MARK_INODE)
-	ret = fanotify_mark(bad_fd, FAN_MARK_ADD | FAN_MARK_INODE,
-		FAN_ACCESS, AT_FDCWD, mounts[0]);
-	(void)ret;
+	VOID_RET(int, fanotify_mark(bad_fd, FAN_MARK_ADD | FAN_MARK_INODE,
+		FAN_ACCESS, AT_FDCWD, mounts[0]));
 #endif
 
 	/* Exercise fanotify_mark by passing two operations simultaneously */
-	ret = fanotify_mark(ret_fd, FAN_MARK_REMOVE | FAN_MARK_ADD,
-		FAN_ACCESS, AT_FDCWD, mounts[0]);
-	(void)ret;
+	VOID_RET(int, fanotify_mark(ret_fd, FAN_MARK_REMOVE | FAN_MARK_ADD,
+		FAN_ACCESS, AT_FDCWD, mounts[0]));
 
 	/* Exercise valid fanotify_mark to increase kernel coverage */
 #if defined(FAN_MARK_INODE)
-	ret = fanotify_mark(ret_fd, FAN_MARK_ADD | FAN_MARK_INODE,
-		FAN_ACCESS, AT_FDCWD, mounts[0]);
-	(void)ret;
+	VOID_RET(int, fanotify_mark(ret_fd, FAN_MARK_ADD | FAN_MARK_INODE,
+		FAN_ACCESS, AT_FDCWD, mounts[0]));
 #endif
 
 #if defined(FAN_MARK_IGNORED_MASK)
-	ret = fanotify_mark(ret_fd, FAN_MARK_ADD | FAN_MARK_IGNORED_MASK,
-		FAN_ACCESS, AT_FDCWD, mounts[0]);
-	(void)ret;
+	VOID_RET(int, fanotify_mark(ret_fd, FAN_MARK_ADD | FAN_MARK_IGNORED_MASK,
+		FAN_ACCESS, AT_FDCWD, mounts[0]));
 #endif
 
 	/* Exercise other invalid combinations of flags */
-	ret = fanotify_mark(ret_fd, FAN_MARK_REMOVE,
-		0, AT_FDCWD, mnts[0]);
-	(void)ret;
+	VOID_RET(int, fanotify_mark(ret_fd, FAN_MARK_REMOVE,
+		0, AT_FDCWD, mnts[0]));
 
 #if defined(FAN_MARK_ONLYDIR)
-	ret = fanotify_mark(ret_fd, FAN_MARK_FLUSH | FAN_MARK_ONLYDIR,
-		FAN_ACCESS, AT_FDCWD, mnts[0]);
-	(void)ret;
+	VOID_RET(int, fanotify_mark(ret_fd, FAN_MARK_FLUSH | FAN_MARK_ONLYDIR,
+		FAN_ACCESS, AT_FDCWD, mnts[0]));
+#endif
+
+#if defined(FAN_MARK_EVICTABLE)
+	VOID_RET(int, fanotify_mark(ret_fd, FAN_MARK_EVICTABLE,
+		FAN_ACCESS, AT_FDCWD, mnts[0]));
 #endif
 
 	(void)close(ret_fd);
@@ -316,7 +328,7 @@ static int test_fanotify_mark(const char *name, char *mounts[])
  *  fanotify_event_init()
  *	initialize fanotify
  */
-static int fanotify_event_init(const char *name, char *mounts[], const int flags)
+static int fanotify_event_init(const char *name, char *mounts[], const unsigned int flags)
 {
 	int fan_fd, count = 0, i;
 
@@ -377,34 +389,27 @@ static void fanotify_event_clear(const int fan_fd)
 	 */
 	for (i = 0; i < n_mnts; i++) {
 		size_t j;
-#if defined(FAN_MARK_MOUNT) || defined(FAN_MARK_FILESYSTEM)
-		int ret;
-#endif
 
 		for (j = 0; j < SIZEOF_ARRAY(fan_stress_settings); j++) {
 #if defined(FAN_MARK_MOUNT)
-			ret = fanotify_mark(fan_fd, FAN_MARK_REMOVE | FAN_MARK_MOUNT,
-				fan_stress_settings[j], AT_FDCWD, mnts[i]);
-			(void)ret;
+			VOID_RET(int, fanotify_mark(fan_fd, FAN_MARK_REMOVE | FAN_MARK_MOUNT,
+				fan_stress_settings[j], AT_FDCWD, mnts[i]));
 #endif
 
 #if defined(FAN_MARK_FILESYSTEM)
-			ret = fanotify_mark(fan_fd, FAN_MARK_REMOVE | FAN_MARK_FILESYSTEM,
-				fan_stress_settings[j], AT_FDCWD, mnts[i]);
-			(void)ret;
+			VOID_RET(int, fanotify_mark(fan_fd, FAN_MARK_REMOVE | FAN_MARK_FILESYSTEM,
+				fan_stress_settings[j], AT_FDCWD, mnts[i]));
 #endif
 		}
 #if defined(FAN_MARK_FLUSH) &&	\
     defined(FAN_MARK_MOUNT)
-		ret = fanotify_mark(fan_fd, FAN_MARK_FLUSH | FAN_MARK_MOUNT,
-				0, AT_FDCWD, mnts[i]);
-		(void)ret;
+		VOID_RET(int, fanotify_mark(fan_fd, FAN_MARK_FLUSH | FAN_MARK_MOUNT,
+				0, AT_FDCWD, mnts[i]));
 #endif
 #if defined(FAN_MARK_FLUSH) &&	\
     defined(FAN_MARK_FILESYSTEM)
-		ret = fanotify_mark(fan_fd, FAN_MARK_FLUSH | FAN_MARK_FILESYSTEM,
-				0, AT_FDCWD, mnts[i]);
-		(void)ret;
+		VOID_RET(int, fanotify_mark(fan_fd, FAN_MARK_FLUSH | FAN_MARK_FILESYSTEM,
+				0, AT_FDCWD, mnts[i]));
 #endif
 	}
 #endif
@@ -462,6 +467,10 @@ static void stress_fanotify_read_events(
 #if defined(FAN_MODIFY)
 			if (metadata->mask & FAN_MODIFY)
 				account->modify++;
+#endif
+#if defined(FAN_RENAME)
+			if (metadata->mask & FAN_RENAME)
+				account->rename++;
 #endif
 			inc_counter(args);
 			(void)close(metadata->fd);
@@ -476,17 +485,20 @@ static void stress_fanotify_read_events(
  */
 static int stress_fanotify(const stress_args_t *args)
 {
-	char pathname[PATH_MAX - 16], filename[PATH_MAX];
-	int ret, pid, rc = EXIT_SUCCESS;
+	char pathname[PATH_MAX - 16], filename[PATH_MAX], filename2[PATH_MAX];
+	const pid_t ppid = getpid();
+	pid_t pid;
+	int ret, rc = EXIT_SUCCESS;
 	stress_fanotify_account_t account;
 
 	(void)memset(&account, 0, sizeof(account));
 
 	stress_temp_dir_args(args, pathname, sizeof(pathname));
 	(void)stress_mk_filename(filename, sizeof(filename), pathname, "fanotify_file");
+	(void)stress_mk_filename(filename2, sizeof(filename2), pathname, "fanotify_file2");
 	ret = stress_temp_dir_mk_args(args);
 	if (ret < 0)
-		return exit_status(-ret);
+		return stress_exit_status(-ret);
 
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
@@ -514,7 +526,6 @@ static int stress_fanotify(const stress_args_t *args)
 
 		do {
 			int fd;
-			ssize_t n;
 			char buffer[64];
 
 			/* Force FAN_CLOSE_NOWRITE */
@@ -522,7 +533,7 @@ static int stress_fanotify(const stress_args_t *args)
 			if (fd < 0) {
 				pr_fail("%s: creat %s failed, errno=%d (%s)\n",
 					args->name, filename, errno, strerror(errno));
-				(void)kill(args->ppid, SIGALRM);
+				(void)kill(ppid, SIGALRM);
 				_exit(EXIT_FAILURE);
 			}
 			(void)close(fd);
@@ -532,11 +543,10 @@ static int stress_fanotify(const stress_args_t *args)
 			if (fd < 0) {
 				pr_fail("%s: open %s O_WRONLY failed, errno=%d (%s)\n",
 					args->name, filename, errno, strerror(errno));
-				(void)kill(args->ppid, SIGALRM);
+				(void)kill(ppid, SIGALRM);
 				_exit(EXIT_FAILURE);
 			}
-			n = write(fd, "test", 4);
-			(void)n;
+			VOID_RET(ssize_t, write(fd, "test", 4));
 			(void)close(fd);
 
 			/* Force FAN_ACCESS */
@@ -544,21 +554,26 @@ static int stress_fanotify(const stress_args_t *args)
 			if (fd < 0) {
 				pr_fail("%s: open %s O_RDONLY failed, errno=%d (%s)\n",
 					args->name, filename, errno, strerror(errno));
-				(void)kill(args->ppid, SIGALRM);
+				(void)kill(ppid, SIGALRM);
 				_exit(EXIT_FAILURE);
 			}
-			n = read(fd, buffer, sizeof(buffer));
-			(void)n;
+			VOID_RET(ssize_t, read(fd, buffer, sizeof(buffer)));
 			(void)close(fd);
 
-			/* Force remove */
-			(void)unlink(filename);
+			if (rename(filename, filename2) < 0)
+				(void)shim_unlink(filename);
+			else
+				(void)shim_unlink(filename2);
+
 		} while (keep_stressing(args));
 
 		_exit(EXIT_SUCCESS);
 	} else {
 		void *buffer;
-		int fan_fd1, max_fd;
+		int fan_fd1;
+#if defined(HAVE_SELECT)
+		int max_fd;
+#endif
 #if defined(FAN_CLASS_NOTIF) &&		\
     defined(FAN_REPORT_DFID_NAME)
 		int fan_fd2;
@@ -587,15 +602,30 @@ static int stress_fanotify(const stress_args_t *args)
 		if (fan_fd2 < 0) {
 			fan_fd2 = -1;
 		}
+#if defined(HAVE_SELECT)
 		max_fd = STRESS_MAXIMUM(fan_fd1, fan_fd2);
+#endif
 #else
+#if defined(HAVE_SELECT)
 		max_fd = fan_fd1;
+#endif
 #endif
 
 		ret = test_fanotify_mark(args->name, mnts);
 		if (ret < 0) {
 			free(buffer);
-			rc = EXIT_FAILURE;
+			switch (-ret) {
+			case EMFILE:
+				pr_inf("%s: too many open files, skipping stressor\n", args->name);
+				rc = EXIT_NO_RESOURCE;
+				break;
+			case ENOMEM:
+				pr_inf("%s: out of memory, skipping stressor\n", args->name);
+				rc = EXIT_NO_RESOURCE;
+				break;
+			default:
+				rc = EXIT_FAILURE;
+			}
 			goto tidy;
 		}
 
@@ -610,6 +640,7 @@ static int stress_fanotify(const stress_args_t *args)
 			if (fan_fd2 >= 0)
 				FD_SET(fan_fd2, &rfds);
 #endif
+#if defined(HAVE_SELECT)
 			ret = select(max_fd + 1, &rfds, NULL, NULL, NULL);
 			if (ret == -1) {
 				if (errno == EINTR)
@@ -620,6 +651,7 @@ static int stress_fanotify(const stress_args_t *args)
 			}
 			if (ret == 0)
 				continue;
+#endif
 
 #if defined(FIONREAD)
 			{
@@ -630,8 +662,7 @@ static int stress_fanotify(const stress_args_t *args)
 				 *  of bytes that are ready to be read
 				 *  for some extra stress
 				 */
-				ret = ioctl(fan_fd1, FIONREAD, &isz);
-				(void)ret;
+				VOID_RET(int, ioctl(fan_fd1, FIONREAD, &isz));
 			}
 #endif
 			if (FD_ISSET(fan_fd1, &rfds))
@@ -666,13 +697,15 @@ static int stress_fanotify(const stress_args_t *args)
 			"%" PRIu64 " close write, "
 			"%" PRIu64 " close nowrite, "
 			"%" PRIu64 " access, "
-			"%" PRIu64 " modify\n",
+			"%" PRIu64 " modify, "
+			"%" PRIu64 " rename\n",
 			args->name,
 			account.open,
 			account.close_write,
 			account.close_nowrite,
 			account.access,
-			account.modify);
+			account.modify,
+			account.rename);
 	}
 tidy:
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
@@ -683,7 +716,7 @@ tidy:
 		(void)kill(pid, SIGKILL);
 		(void)shim_waitpid(pid, &status, 0);
 	}
-	(void)unlink(filename);
+	(void)shim_unlink(filename);
 	(void)stress_temp_dir_rm_args(args);
 	stress_mount_free(mnts, n_mnts);
 
@@ -694,6 +727,7 @@ stressor_info_t stress_fanotify_info = {
 	.stressor = stress_fanotify,
 	.supported = stress_fanotify_supported,
 	.class = CLASS_FILESYSTEM | CLASS_SCHEDULER | CLASS_OS,
+	.verify = VERIFY_ALWAYS,
 	.help = help
 };
 #else
