@@ -311,11 +311,15 @@ static inline int stress_io_uring_complete(
 			break;
 
 		cqe = &cring->cqes[head & *submit->cq_ring.ring_mask];
-		if ((cqe->res < 0) && (opcode != IORING_OP_FALLOCATE)) {
+		if ((cqe->res < 0)
+#ifdef HAVE_IORING_OP_FALLOCATE
+			&& (opcode != IORING_OP_FALLOCATE)
+#endif
+		) {
 			const int err = abs(cqe->res);
 
-			/* Silently ignore EOPNOTSUPP completion errors */
-			if (errno == EOPNOTSUPP) {
+			/* Silently ignore some completion errors */
+			if ((err == EOPNOTSUPP) || (err == ENOTDIR)) {
 				*supported = false;
 			} else  {
 				pr_fail("%s: completion opcode=%d (%s), error=%d (%s)\n",
@@ -769,7 +773,7 @@ static int stress_io_uring(const stress_args_t *args)
 	ret = stress_temp_dir_mk_args(args);
 	if (ret < 0) {
 		stress_io_uring_unmap_iovecs(&io_uring_file);
-		return exit_status(-ret);
+		return stress_exit_status(-ret);
 	}
 
 	(void)stress_temp_filename_args(args,
@@ -782,7 +786,7 @@ static int stress_io_uring(const stress_args_t *args)
 		goto clean;
 
 	if ((io_uring_file.fd = open(filename, O_CREAT | O_RDWR | O_DSYNC, S_IRUSR | S_IWUSR)) < 0) {
-		rc = exit_status(errno);
+		rc = stress_exit_status(errno);
 		pr_fail("%s: open on %s failed, errno=%d (%s)\n",
 			args->name, filename, errno, strerror(errno));
 		goto clean;
@@ -795,7 +799,7 @@ static int stress_io_uring(const stress_args_t *args)
 	 *  Assume all opcodes are supported
 	 */
 	for (j = 0; j < SIZEOF_ARRAY(stress_io_uring_setups); j++) {
-		supported[i] = true;
+		supported[j] = true;
 	}
 
 	rc = EXIT_SUCCESS;
