@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyrignt (C) 2021-2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,16 +16,22 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
+#include "core-put.h"
 
-typedef void (*stress_iomix_func)(const stress_args_t *args, const int fd, const off_t iomix_bytes);
+#if defined(HAVE_LINUX_FS_H)
+#include <linux/fs.h>
+#endif
+#if defined(HAVE_SYS_SENDFILE_H)
+#include <sys/sendfile.h>
+#endif
+
+#define MIN_IOMIX_BYTES		(1 * MB)
+#define MAX_IOMIX_BYTES		(MAX_FILE_LIMIT)
+#define DEFAULT_IOMIX_BYTES	(1 * GB)
+
+typedef void (*stress_iomix_func)(const stress_args_t *args, const int fd, const char *fs_type, const off_t iomix_bytes);
 
 static const stress_help_t help[] = {
 	{ NULL,	"iomix N",	 "start N workers that have a mix of I/O operations" },
@@ -33,6 +40,8 @@ static const stress_help_t help[] = {
 	{ NULL, NULL,		 NULL }
 };
 
+static void *counter_lock;
+
 static int stress_set_iomix_bytes(const char *opt)
 {
 	off_t iomix_bytes;
@@ -49,7 +58,35 @@ static int stress_set_iomix_bytes(const char *opt)
  */
 static off_t stress_iomix_rnd_offset(const off_t max)
 {
-	return (off_t)(stress_mwc64() % max);
+	return (off_t)(stress_mwc64() % (uint64_t)max);
+}
+
+/*
+ *  stress_iomix_fadvise_random_dontneed()
+ *	hint that the data at offset is not needed
+ *	and the I/O is random for more stress
+ */
+static void stress_iomix_fadvise_random_dontneed(
+	const int fd,
+	const off_t offset,
+	const off_t len)
+{
+#if defined(HAVE_POSIX_FADVISE) &&	\
+    (defined(POSIX_FADV_RANDOM) ||	\
+     defined(POSIX_FADV_DONTNEED))
+	int flag = 0;
+#if defined(POSIX_FADV_RANDOM)
+	flag |= POSIX_FADV_RANDOM;
+#endif
+#if defined(POSIX_FADV_DONTNEED)
+	flag |= POSIX_FADV_DONTNEED;
+#endif
+	(void)posix_fadvise(fd, offset, len, flag);
+#else
+	(void)fd;
+	(void)offset;
+	(void)len;
+#endif
 }
 
 /*
@@ -100,25 +137,27 @@ static void stress_iomix_fsync_min_1Hz(const int fd)
 static void stress_iomix_wr_seq_bursts(
 	const stress_args_t *args,
 	const int fd,
+	const char *fs_type,
 	const off_t iomix_bytes)
 {
 	do {
 		off_t ret, posn;
 		const int n = stress_mwc8();
 		int i;
-		struct timeval tv;
 
 		posn = stress_iomix_rnd_offset(iomix_bytes);
 		ret = lseek(fd, posn, SEEK_SET);
-		if (ret < 0) {
-			pr_fail("%s: lseek failed, errno=%d (%s)\n",
-				args->name, errno, strerror(errno));
+		if (ret == (off_t)-1) {
+			pr_fail("%s: lseek failed, errno=%d (%s)%s\n",
+				args->name, errno, strerror(errno), fs_type);
 			return;
 		}
 #if defined(HAVE_POSIX_FADVISE) &&      \
     defined(POSIX_FADV_SEQUENTIAL)
 		if (posn < iomix_bytes)
 			(void)posix_fadvise(fd, posn, iomix_bytes - posn, POSIX_FADV_SEQUENTIAL);
+#else
+		UNEXPECTED
 #endif
 		for (i = 0; (i < n) && (posn < iomix_bytes); i++) {
 			char buffer[512];
@@ -130,21 +169,19 @@ static void stress_iomix_wr_seq_bursts(
 			rc = write(fd, buffer, len);
 			if (rc < 0) {
 				if (errno != EPERM) {
-					pr_fail("%s: write failed, errno=%d (%s)\n",
-						args->name, errno, strerror(errno));
+					pr_fail("%s: write failed, errno=%d (%s)%s\n",
+						args->name, errno, strerror(errno), fs_type);
 					return;
 				}
 			}
 			posn += rc;
-			if (!keep_stressing(args))
+
+			if (!inc_counter_lock(args, counter_lock, true))
 				return;
-			inc_counter(args);
 			stress_iomix_fsync_min_1Hz(fd);
 		}
-		tv.tv_sec = 0;
-		tv.tv_usec = stress_mwc32() % 1000000;
-		(void)select(0, NULL, NULL, NULL, &tv);
-	} while (keep_stressing(args));
+		shim_usleep(stress_mwc32() % 1000000);
+	} while (inc_counter_lock(args, counter_lock, false));
 }
 
 /*
@@ -154,16 +191,18 @@ static void stress_iomix_wr_seq_bursts(
 static void stress_iomix_wr_rnd_bursts(
 	const stress_args_t *args,
 	const int fd,
+	const char *fs_type,
 	const off_t iomix_bytes)
 {
 #if defined(HAVE_POSIX_FADVISE) &&      \
     defined(POSIX_FADV_RANDOM)
 	(void)posix_fadvise(fd, 0, iomix_bytes, POSIX_FADV_RANDOM);
+#else
+	UNEXPECTED
 #endif
 	do {
 		const int n = stress_mwc8();
 		int i;
-		struct timeval tv;
 
 		for (i = 0; i < n; i++) {
 			char buffer[512];
@@ -173,9 +212,9 @@ static void stress_iomix_wr_rnd_bursts(
 
 			posn = stress_iomix_rnd_offset(iomix_bytes);
 			ret = lseek(fd, posn, SEEK_SET);
-			if (ret < 0) {
-				pr_fail("%s: lseek failed, errno=%d (%s)\n",
-					args->name, errno, strerror(errno));
+			if (ret == (off_t)-1) {
+				pr_fail("%s: lseek failed, errno=%d (%s)%s\n",
+					args->name, errno, strerror(errno), fs_type);
 				return;
 			}
 
@@ -183,21 +222,17 @@ static void stress_iomix_wr_rnd_bursts(
 			rc = write(fd, buffer, len);
 			if (rc < 0) {
 				if (errno != EPERM) {
-					pr_fail("%s: write failed, errno=%d (%s)\n",
-						args->name, errno, strerror(errno));
+					pr_fail("%s: write failed, errno=%d (%s)%s\n",
+						args->name, errno, strerror(errno), fs_type);
 					return;
 				}
 			}
-			if (!keep_stressing(args))
+			if (!inc_counter_lock(args, counter_lock, true))
 				return;
-			inc_counter(args);
 			stress_iomix_fsync_min_1Hz(fd);
 		}
-		tv.tv_sec = stress_mwc32() % 2;
-		tv.tv_usec = stress_mwc32() % 1000000;
-		(void)select(0, NULL, NULL, NULL, &tv);
-
-	} while (keep_stressing(args));
+		shim_usleep(stress_mwc32() % 2000000);
+	} while (inc_counter_lock(args, counter_lock, false));
 }
 
 /*
@@ -207,21 +242,24 @@ static void stress_iomix_wr_rnd_bursts(
 static void stress_iomix_wr_seq_slow(
 	const stress_args_t *args,
 	const int fd,
+	const char *fs_type,
 	const off_t iomix_bytes)
 {
 	do {
 		off_t ret, posn = 0;
 
 		ret = lseek(fd, 0, SEEK_SET);
-		if (ret < 0) {
-			pr_fail("%s: lseek failed, errno=%d (%s)\n",
-				args->name, errno, strerror(errno));
+		if (ret == (off_t)-1) {
+			pr_fail("%s: lseek failed, errno=%d (%s)%s\n",
+				args->name, errno, strerror(errno), fs_type);
 			return;
 		}
 #if defined(HAVE_POSIX_FADVISE) &&      \
     defined(POSIX_FADV_SEQUENTIAL)
 		if (posn < iomix_bytes)
 			(void)posix_fadvise(fd, posn, iomix_bytes - posn, POSIX_FADV_SEQUENTIAL);
+#else
+		UNEXPECTED
 #endif
 		while (posn < iomix_bytes) {
 			char buffer[512];
@@ -233,19 +271,18 @@ static void stress_iomix_wr_seq_slow(
 			rc = write(fd, buffer, len);
 			if (rc < 0) {
 				if (errno != EPERM) {
-					pr_fail("%s: write failed, errno=%d (%s)\n",
-						args->name, errno, strerror(errno));
+					pr_fail("%s: write failed, errno=%d (%s)%s\n",
+						args->name, errno, strerror(errno), fs_type);
 					return;
 				}
 			}
 			(void)shim_usleep(250000);
 			posn += rc;
-			if (!keep_stressing(args))
+			if (!inc_counter_lock(args, counter_lock, true))
 				return;
-			inc_counter(args);
 			stress_iomix_fsync_min_1Hz(fd);
 		}
-	} while (keep_stressing(args));
+	} while (inc_counter_lock(args, counter_lock, false));
 }
 
 /*
@@ -255,19 +292,19 @@ static void stress_iomix_wr_seq_slow(
 static void stress_iomix_rd_seq_bursts(
 	const stress_args_t *args,
 	const int fd,
+	const char *fs_type,
 	const off_t iomix_bytes)
 {
 	do {
 		off_t ret, posn;
 		const int n = stress_mwc8();
 		int i;
-		struct timeval tv;
 
 		posn = stress_iomix_rnd_offset(iomix_bytes);
 		ret = lseek(fd, posn, SEEK_SET);
-		if (ret < 0) {
-			pr_fail("%s: lseek failed, errno=%d (%s)\n",
-				args->name, errno, strerror(errno));
+		if (ret == (off_t)-1) {
+			pr_fail("%s: lseek failed, errno=%d (%s)%s\n",
+				args->name, errno, strerror(errno), fs_type);
 			return;
 		}
 
@@ -275,6 +312,8 @@ static void stress_iomix_rd_seq_bursts(
     defined(POSIX_FADV_SEQUENTIAL)
 		if (posn < iomix_bytes)
 			(void)posix_fadvise(fd, posn, iomix_bytes - posn, POSIX_FADV_SEQUENTIAL);
+#else
+		UNEXPECTED
 #endif
 		for (i = 0; (i < n) && (posn < iomix_bytes); i++) {
 			char buffer[512];
@@ -283,19 +322,19 @@ static void stress_iomix_rd_seq_bursts(
 
 			rc = read(fd, buffer, len);
 			if (rc < 0) {
-				pr_fail("%s: read failed, errno=%d (%s)\n",
-					args->name, errno, strerror(errno));
+				pr_fail("%s: read failed, errno=%d (%s)%s\n",
+					args->name, errno, strerror(errno), fs_type);
 				return;
 			}
 			posn += rc;
-			if (!keep_stressing(args))
+			if (!inc_counter_lock(args, counter_lock, true))
 				return;
-			inc_counter(args);
+
+			/* Add some unhelpful advice */
+			stress_iomix_fadvise_random_dontneed(fd, posn, 4096);
 		}
-		tv.tv_sec = 0;
-		tv.tv_usec = stress_mwc32() % 1000000;
-		(void)select(0, NULL, NULL, NULL, &tv);
-	} while (keep_stressing(args));
+		shim_usleep(stress_mwc32() % 1000000);
+	} while (inc_counter_lock(args, counter_lock, false));
 }
 
 /*
@@ -305,12 +344,12 @@ static void stress_iomix_rd_seq_bursts(
 static void stress_iomix_rd_rnd_bursts(
 	const stress_args_t *args,
 	const int fd,
+	const char *fs_type,
 	const off_t iomix_bytes)
 {
 	do {
 		const int n = stress_mwc8();
 		int i;
-		struct timeval tv;
 
 		for (i = 0; i < n; i++) {
 			char buffer[512];
@@ -319,31 +358,28 @@ static void stress_iomix_rd_rnd_bursts(
 			off_t ret, posn;
 
 			posn = stress_iomix_rnd_offset(iomix_bytes);
-#if defined(HAVE_POSIX_FADVISE) &&      \
-    defined(POSIX_FADV_RANDOM)
-			(void)posix_fadvise(fd, posn, (off_t)len, POSIX_FADV_RANDOM);
-#endif
+
+			/* Add some unhelpful advice */
+			stress_iomix_fadvise_random_dontneed(fd, posn, (ssize_t)len);
+
 			ret = lseek(fd, posn, SEEK_SET);
-			if (ret < 0) {
-				pr_fail("%s: lseek failed, errno=%d (%s)\n",
-					args->name, errno, strerror(errno));
+			if (ret == (off_t)-1) {
+				pr_fail("%s: lseek failed, errno=%d (%s)%s\n",
+					args->name, errno, strerror(errno), fs_type);
 				return;
 			}
 
 			rc = read(fd, buffer, len);
 			if (rc < 0) {
-				pr_fail("%s: read failed, errno=%d (%s)\n",
-					args->name, errno, strerror(errno));
+				pr_fail("%s: read failed, errno=%d (%s)%s\n",
+					args->name, errno, strerror(errno), fs_type);
 				return;
 			}
-			if (!keep_stressing(args))
+			if (!inc_counter_lock(args, counter_lock, true))
 				return;
-			inc_counter(args);
 		}
-		tv.tv_sec = stress_mwc32() % 3;
-		tv.tv_usec = stress_mwc32() % 1000000;
-		(void)select(0, NULL, NULL, NULL, &tv);
-	} while (keep_stressing(args));
+		shim_usleep(3000000);
+	} while (inc_counter_lock(args, counter_lock, false));
 }
 
 /*
@@ -353,41 +389,46 @@ static void stress_iomix_rd_rnd_bursts(
 static void stress_iomix_rd_seq_slow(
 	const stress_args_t *args,
 	const int fd,
+	const char *fs_type,
 	const off_t iomix_bytes)
 {
 	do {
 		off_t ret, posn = 0;
 
 		ret = lseek(fd, 0, SEEK_SET);
-		if (ret < 0) {
-			pr_fail("%s: lseek failed, errno=%d (%s)\n",
-				args->name, errno, strerror(errno));
+		if (ret == (off_t)-1) {
+			pr_fail("%s: lseek failed, errno=%d (%s)%s\n",
+				args->name, errno, strerror(errno), fs_type);
 			return;
 		}
 #if defined(HAVE_POSIX_FADVISE) &&      \
     defined(POSIX_FADV_SEQUENTIAL)
 		if (posn < iomix_bytes)
 			(void)posix_fadvise(fd, posn, iomix_bytes - posn, POSIX_FADV_SEQUENTIAL);
+#else
+		UNEXPECTED
 #endif
 		while (posn < iomix_bytes) {
 			char buffer[512];
 			ssize_t rc;
 			const size_t len = 1 + (stress_mwc32() & (sizeof(buffer) - 1));
 
+			/* Add some unhelpful advice */
+			stress_iomix_fadvise_random_dontneed(fd, posn, (ssize_t)len);
+
 			rc = read(fd, buffer, len);
 			if (rc < 0) {
-				pr_fail("%s: read failed, errno=%d (%s)\n",
-					args->name, errno, strerror(errno));
+				pr_fail("%s: read failed, errno=%d (%s)%s\n",
+					args->name, errno, strerror(errno), fs_type);
 				return;
 			}
 			(void)shim_usleep(333333);
 			posn += rc;
-			if (!keep_stressing(args))
+			if (!inc_counter_lock(args, counter_lock, true))
 				return;
-			inc_counter(args);
 			stress_iomix_fsync_min_1Hz(fd);
 		}
-	} while (keep_stressing(args));
+	} while (inc_counter_lock(args, counter_lock, false));
 }
 
 /*
@@ -397,46 +438,49 @@ static void stress_iomix_rd_seq_slow(
 static void stress_iomix_sync(
 	const stress_args_t *args,
 	const int fd,
+	const char *fs_type,
 	const off_t iomix_bytes)
 {
-	do {
-		struct timeval tv;
+	(void)fs_type;
 
+	do {
 		(void)shim_fsync(fd);
-		if (!keep_stressing(args))
+		if (!inc_counter_lock(args, counter_lock, true))
 			break;
-		inc_counter(args);
-		tv.tv_sec = stress_mwc32() % 4;
-		tv.tv_usec = stress_mwc32() % 1000000;
-		(void)select(0, NULL, NULL, NULL, &tv);
-		if (!keep_stressing(args))
+		shim_usleep(stress_mwc32() % 4000000);
+		if (!inc_counter_lock(args, counter_lock, false))
 			break;
 
 #if defined(HAVE_FDATASYNC)
 		(void)shim_fdatasync(fd);
 		/* Exercise illegal fdatasync */
 		(void)shim_fdatasync(-1);
-		if (!keep_stressing(args))
+		if (!inc_counter_lock(args, counter_lock, false))
 			break;
-		tv.tv_sec = stress_mwc32() % 4;
-		tv.tv_usec = stress_mwc32() % 1000000;
-		(void)select(0, NULL, NULL, NULL, &tv);
-		if (!keep_stressing(args))
+		shim_usleep(stress_mwc32() % 4000000);
+		if (!inc_counter_lock(args, counter_lock, false))
 			break;
+#else
+		UNEXPECTED
 #endif
 #if defined(HAVE_SYNC_FILE_RANGE) &&	\
     defined(SYNC_FILE_RANGE_WRITE)
-		(void)sync_file_range(fd, stress_iomix_rnd_offset(iomix_bytes),
-				65536, SYNC_FILE_RANGE_WRITE);
-		if (!keep_stressing(args))
-			break;
-		tv.tv_sec = stress_mwc32() % 4;
-		tv.tv_usec = stress_mwc32() % 1000000;
-		(void)select(0, NULL, NULL, NULL, &tv);
+		{
+			const off_t posn = stress_iomix_rnd_offset(iomix_bytes);
+
+			(void)sync_file_range(fd, posn, 65536,
+				SYNC_FILE_RANGE_WRITE);
+			stress_iomix_fadvise_random_dontneed(fd, posn, 65536);
+
+			if (!inc_counter_lock(args, counter_lock, false))
+				break;
+			shim_usleep(stress_mwc32() % 4000000);
+		}
 #else
 		(void)iomix_bytes;
+		UNEXPECTED
 #endif
-	} while (keep_stressing(args));
+	} while (inc_counter_lock(args, counter_lock, false));
 }
 
 #if defined(HAVE_POSIX_FADVISE) &&	\
@@ -448,8 +492,11 @@ static void stress_iomix_sync(
 static void stress_iomix_bad_advise(
 	const stress_args_t *args,
 	const int fd,
+	const char *fs_type,
 	const off_t iomix_bytes)
 {
+	(void)fs_type;
+
 	do {
 		off_t posn = stress_iomix_rnd_offset(iomix_bytes);
 
@@ -457,7 +504,7 @@ static void stress_iomix_bad_advise(
 		(void)shim_usleep(100000);
 		(void)posix_fadvise(fd, posn, 65536, POSIX_FADV_NORMAL);
 		(void)shim_usleep(100000);
-	} while (keep_stressing(args));
+	} while (inc_counter_lock(args, counter_lock, true));
 }
 #endif
 
@@ -468,6 +515,7 @@ static void stress_iomix_bad_advise(
 static void stress_iomix_rd_wr_mmap(
 	const stress_args_t *args,
 	const int fd,
+	const char *fs_type,
 	const off_t iomix_bytes)
 {
 	void *mmaps[128];
@@ -478,6 +526,7 @@ static void stress_iomix_rd_wr_mmap(
 #if defined(MAP_POPULATE)
 	flags |= MAP_POPULATE;
 #endif
+	(void)fs_type;
 
 	do {
 		for (i = 0; i < SIZEOF_ARRAY(mmaps); i++) {
@@ -507,7 +556,7 @@ static void stress_iomix_rd_wr_mmap(
 			if (mmaps[i] != MAP_FAILED)
 				(void)munmap(mmaps[i], page_size);
 		}
-	} while (keep_stressing(args));
+	} while (inc_counter_lock(args, counter_lock, true));
 }
 
 /*
@@ -517,15 +566,16 @@ static void stress_iomix_rd_wr_mmap(
 static void stress_iomix_wr_bytes(
 	const stress_args_t *args,
 	const int fd,
+	const char *fs_type,
 	const off_t iomix_bytes)
 {
 	do {
 		off_t ret, posn = 0;
 
 		ret = lseek(fd, 0, SEEK_SET);
-		if (ret < 0) {
-			pr_fail("%s: lseek failed, errno=%d (%s)\n",
-				args->name, errno, strerror(errno));
+		if (ret == (off_t)-1) {
+			pr_fail("%s: lseek failed, errno=%d (%s)%s\n",
+				args->name, errno, strerror(errno), fs_type);
 			return;
 		}
 		while (posn < iomix_bytes) {
@@ -535,18 +585,17 @@ static void stress_iomix_wr_bytes(
 			rc = write(fd, buffer, sizeof(buffer));
 			if (rc < 0) {
 				if (errno != EPERM) {
-					pr_fail("%s: write failed, errno=%d (%s)\n",
-						args->name, errno, strerror(errno));
+					pr_fail("%s: write failed, errno=%d (%s)%s\n",
+						args->name, errno, strerror(errno), fs_type);
 					return;
 			}	}
 			(void)shim_usleep(1000);
 			posn += rc;
-			if (!keep_stressing(args))
+			if (!inc_counter_lock(args, counter_lock, true))
 				return;
-			inc_counter(args);
 			stress_iomix_fsync_min_1Hz(fd);
 		}
-	} while (keep_stressing(args));
+	} while (inc_counter_lock(args, counter_lock, false));
 }
 
 /*
@@ -556,15 +605,16 @@ static void stress_iomix_wr_bytes(
 static void stress_iomix_wr_rev_bytes(
 	const stress_args_t *args,
 	const int fd,
+	const char *fs_type,
 	const off_t iomix_bytes)
 {
 	do {
 		off_t ret, posn = iomix_bytes;
 
 		ret = lseek(fd, 0, SEEK_SET);
-		if (ret < 0) {
-			pr_fail("%s: lseek failed, errno=%d (%s)\n",
-				args->name, errno, strerror(errno));
+		if (ret == (off_t)-1) {
+			pr_fail("%s: lseek failed, errno=%d (%s)%s\n",
+				args->name, errno, strerror(errno), fs_type);
 			return;
 		}
 		while (posn != 0) {
@@ -574,18 +624,17 @@ static void stress_iomix_wr_rev_bytes(
 			rc = write(fd, buffer, sizeof(buffer));
 			if (rc < 0) {
 				if (errno != EPERM) {
-					pr_fail("%s: write failed, errno=%d (%s)\n",
-						args->name, errno, strerror(errno));
+					pr_fail("%s: write failed, errno=%d (%s)%s\n",
+						args->name, errno, strerror(errno), fs_type);
 					return;
 			}	}
 			(void)shim_usleep(1000);
 			posn--;
-			if (!keep_stressing(args))
+			if (!inc_counter_lock(args, counter_lock, true))
 				return;
-			inc_counter(args);
 			stress_iomix_fsync_min_1Hz(fd);
 		}
-	} while (keep_stressing(args));
+	} while (inc_counter_lock(args, counter_lock, false));
 }
 
 /*
@@ -595,6 +644,7 @@ static void stress_iomix_wr_rev_bytes(
 static void stress_iomix_rd_bytes(
 	const stress_args_t *args,
 	const int fd,
+	const char *fs_type,
 	const off_t iomix_bytes)
 {
 	do {
@@ -604,28 +654,30 @@ static void stress_iomix_rd_bytes(
 			char buffer[1];
 			ssize_t rc;
 
+			/* Add some unhelpful advice */
+			stress_iomix_fadvise_random_dontneed(fd, posn, sizeof(buffer));
+
 			ret = lseek(fd, posn, SEEK_SET);
-			if (ret < 0) {
-				pr_fail("%s: lseek failed, errno=%d (%s)\n",
-					args->name, errno, strerror(errno));
+			if (ret == (off_t)-1) {
+				pr_fail("%s: lseek failed, errno=%d (%s)%s\n",
+					args->name, errno, strerror(errno), fs_type);
 				return;
 			}
 
 			rc = read(fd, buffer, sizeof(buffer));
 			if (rc < 0) {
 				if (errno != EPERM) {
-					pr_fail("%s: write failed, errno=%d (%s)\n",
-						args->name, errno, strerror(errno));
+					pr_fail("%s: write failed, errno=%d (%s)%s\n",
+						args->name, errno, strerror(errno), fs_type);
 					return;
 				}
 			}
 			(void)shim_usleep(1000);
 			posn--;
-			if (!keep_stressing(args))
+			if (!inc_counter_lock(args, counter_lock, true))
 				return;
-			inc_counter(args);
 		}
-	} while (keep_stressing(args));
+	} while (inc_counter_lock(args, counter_lock, false));
 }
 
 #if defined(__linux__)
@@ -670,7 +722,11 @@ static void stress_iomix_inode_ioctl(
 	ret = ioctl(fd, FS_IOC_SETFLAGS, &attr);
 	if (ret < 0)
 		return;
+#else
+	UNEXPECTED
 #endif
+#else
+	UNEXPECTED
 #endif
 	*ok = true;
 }
@@ -683,51 +739,75 @@ static void stress_iomix_inode_ioctl(
 static void stress_iomix_inode_flags(
 	const stress_args_t *args,
 	const int fd,
+	const char *fs_type,
 	const off_t iomix_bytes)
 {
 	(void)args;
 	(void)fd;
+	(void)fs_type;
 	(void)iomix_bytes;
 
 	do {
 		bool ok = false;
 #if defined(FS_APPEND_FL)
 		stress_iomix_inode_ioctl(args, fd, FS_APPEND_FL, &ok);
+#else
+		UNEXPECTED
 #endif
 #if defined(FS_COMPR_FL)
 		stress_iomix_inode_ioctl(args, fd, FS_COMPR_FL, &ok);
+#else
+		UNEXPECTED
 #endif
 #if defined(FS_IMMUTABLE_FL)
 		stress_iomix_inode_ioctl(args, fd, FS_IMMUTABLE_FL, &ok);
+#else
+		UNEXPECTED
 #endif
 #if defined(FS_JOURNAL_DATA_FL)
 		stress_iomix_inode_ioctl(args, fd, FS_JOURNAL_DATA_FL, &ok);
+#else
+		UNEXPECTED
 #endif
 #if defined(FS_NOATIME_FL)
 		stress_iomix_inode_ioctl(args, fd, FS_NOATIME_FL, &ok);
+#else
+		UNEXPECTED
 #endif
 #if defined(FS_NOCOW_FL)
 		stress_iomix_inode_ioctl(args, fd, FS_NOCOW_FL, &ok);
+#else
+		UNEXPECTED
 #endif
 #if defined(FS_NODUMP_FL)
 		stress_iomix_inode_ioctl(args, fd, FS_NODUMP_FL, &ok);
+#else
+		UNEXPECTED
 #endif
 #if defined(FS_NOTAIL_FL)
 		stress_iomix_inode_ioctl(args, fd, FS_NOTAIL_FL, &ok);
+#else
+		UNEXPECTED
 #endif
 #if defined(FS_SECRM_FL)
 		stress_iomix_inode_ioctl(args, fd, FS_SECRM_FL, &ok);
+#else
+		UNEXPECTED
 #endif
 #if defined(FS_SYNC_FL)
 		stress_iomix_inode_ioctl(args, fd, FS_SYNC_FL, &ok);
+#else
+		UNEXPECTED
 #endif
 #if defined(FS_UNRM_FL)
 		stress_iomix_inode_ioctl(args, fd, FS_UNRM_FL, &ok);
+#else
+		UNEXPECTED
 #endif
 		if (!ok)
 			_exit(EXIT_SUCCESS);
 		stress_iomix_fsync_min_1Hz(fd);
-	} while (keep_stressing(args));
+	} while (inc_counter_lock(args, counter_lock, true));
 }
 #endif
 
@@ -739,9 +819,11 @@ static void stress_iomix_inode_flags(
 static void stress_iomix_drop_caches(
 	const stress_args_t *args,
 	const int fd,
+	const char *fs_type,
 	const off_t iomix_bytes)
 {
 	(void)fd;
+	(void)fs_type;
 	(void)iomix_bytes;
 
 	do {
@@ -761,7 +843,74 @@ static void stress_iomix_drop_caches(
 		if (system_write("/proc/sys/vm/drop_caches", "3", 1) < 0)
 			(void)pause();
 		(void)sleep(5);
-	} while (keep_stressing(args));
+	} while (inc_counter_lock(args, counter_lock, true));
+}
+#endif
+
+#if defined(HAVE_COPY_FILE_RANGE)
+/*
+ *  stress_iomix_copy_file_range()
+ *	lots of copies with copy_file_range
+ */
+static void stress_iomix_copy_file_range(
+	const stress_args_t *args,
+	const int fd,
+	const char *fs_type,
+	const off_t iomix_bytes)
+{
+	(void)fs_type;
+
+	do {
+		off_t from = stress_iomix_rnd_offset(iomix_bytes);
+		off_t to = stress_iomix_rnd_offset(iomix_bytes);
+		const size_t size = stress_mwc16();
+
+		VOID_RET(ssize_t, copy_file_range(fd, &from, fd, &to, size, 0));
+		VOID_RET(ssize_t, copy_file_range(fd, &to, fd, &from, size, 0));
+
+		if (!keep_stressing(args))
+			return;
+		stress_iomix_fsync_min_1Hz(fd);
+
+		shim_usleep(stress_mwc32() % 100000);
+	} while (inc_counter_lock(args, counter_lock, true));
+}
+#endif
+
+#if defined(HAVE_SYS_SENDFILE_H) &&	\
+    defined(HAVE_SENDFILE)
+/*
+ *  stress_iomix_sendfile()
+ *	lots of copies with copy_file_range
+ */
+static void stress_iomix_sendfile(
+	const stress_args_t *args,
+	const int fd,
+	const char *fs_type,
+	const off_t iomix_bytes)
+{
+	(void)fs_type;
+
+	do {
+		off_t from = stress_iomix_rnd_offset(iomix_bytes);
+		off_t to = stress_iomix_rnd_offset(iomix_bytes);
+		off_t ret;
+		const size_t size = stress_mwc16();
+
+		ret = lseek(fd, to, SEEK_SET);
+		if (ret != (off_t)-1) {
+			ssize_t sret;
+
+			sret = sendfile(fd, fd, &from, size);
+			(void)sret;
+		}
+
+		if (!keep_stressing(args))
+			return;
+		stress_iomix_fsync_min_1Hz(fd);
+
+		shim_usleep(stress_mwc32() % 130000);
+	} while (inc_counter_lock(args, counter_lock, true));
 }
 #endif
 
@@ -786,7 +935,14 @@ static stress_iomix_func iomix_funcs[] = {
 	stress_iomix_inode_flags,
 #endif
 #if defined(__linux__)
-	stress_iomix_drop_caches
+	stress_iomix_drop_caches,
+#endif
+#if defined(HAVE_COPY_FILE_RANGE)
+	stress_iomix_copy_file_range,
+#endif
+#if defined(HAVE_SYS_SENDFILE_H) &&	\
+    defined(HAVE_SENDFILE)
+	stress_iomix_sendfile,
 #endif
 };
 
@@ -798,19 +954,21 @@ static int stress_iomix(const stress_args_t *args)
 {
 	int fd, ret;
 	char filename[PATH_MAX];
-	uint64_t *counters;
 	off_t iomix_bytes = DEFAULT_IOMIX_BYTES;
 	const size_t page_size = args->page_size;
-	const size_t counters_sz = sizeof(uint64_t) * SIZEOF_ARRAY(iomix_funcs);
-	const size_t sz = (counters_sz + page_size) & ~(page_size - 1);
 	size_t i;
 	int pids[SIZEOF_ARRAY(iomix_funcs)];
+	const char *fs_type;
+	int oflags = O_CREAT | O_RDWR;
+	const pid_t parent = getpid();
 
-	counters = (void *)mmap(NULL, sz, PROT_READ | PROT_WRITE,
-			MAP_SHARED | MAP_ANONYMOUS, -1, 0);
-	if (counters == MAP_FAILED) {
-		pr_fail("%s: mmap failed, errno=%d (%s)\n",
-			args->name, errno, strerror(errno));
+#if defined(O_SYNC)
+	oflags |= O_SYNC;
+#endif
+
+	counter_lock = stress_lock_create();
+	if (!counter_lock) {
+		pr_inf("%s: failed to create counter lock. skipping stressor\n", args->name);
 		return EXIT_NO_RESOURCE;
 	}
 
@@ -828,19 +986,20 @@ static int stress_iomix(const stress_args_t *args)
 
 	ret = stress_temp_dir_mk_args(args);
 	if (ret < 0) {
-		ret = exit_status(-ret);
-		goto unmap;
+		ret = stress_exit_status(-ret);
+		goto lock_destroy;
 	}
 
 	(void)stress_temp_filename_args(args,
 		filename, sizeof(filename), stress_mwc32());
-	if ((fd = open(filename, O_CREAT | O_RDWR | O_SYNC, S_IRUSR | S_IWUSR)) < 0) {
-		ret = exit_status(errno);
+	if ((fd = open(filename, oflags, S_IRUSR | S_IWUSR)) < 0) {
+		ret = stress_exit_status(errno);
 		pr_fail("%s: open %s failed, errno=%d (%s)\n",
 			args->name, filename, errno, strerror(errno));
-		goto unmap;
+		goto lock_destroy;
 	}
-	(void)unlink(filename);
+	fs_type = stress_fs_type(filename);
+	(void)shim_unlink(filename);
 
 #if defined(FALLOC_FL_ZERO_RANGE)
 	ret = shim_fallocate(fd, FALLOC_FL_ZERO_RANGE, 0, iomix_bytes);
@@ -852,51 +1011,36 @@ static int stress_iomix(const stress_args_t *args)
 			ret = EXIT_NO_RESOURCE;
 		} else {
 			ret = EXIT_FAILURE;
-			pr_fail("%s: fallocate failed, errno=%d (%s)\n",
-				args->name, errno, strerror(errno));
+			pr_fail("%s: fallocate failed, errno=%d (%s)%s\n",
+				args->name, errno, strerror(errno), fs_type);
 		}
 		goto tidy;
 	}
 
 	(void)memset(pids, 0, sizeof(pids));
-	(void)memset(counters, 0, sz);
 
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
 	for (i = 0; i < SIZEOF_ARRAY(iomix_funcs); i++) {
-		stress_args_t tmp_args = *args;
-
-		tmp_args.counter = &counters[i];
-
 		pids[i] = fork();
 		if (pids[i] < 0) {
 			goto reap;
 		} else if (pids[i] == 0) {
 			/* Child */
 			(void)sched_settings_apply(true);
-			iomix_funcs[i](&tmp_args, fd, iomix_bytes);
+			iomix_funcs[i](args, fd, fs_type, iomix_bytes);
+			(void)kill(parent, SIGALRM);
 			_exit(EXIT_SUCCESS);
 		}
 	}
 
 	do {
-		uint64_t c = 0;
-		(void)shim_usleep(5000);
-		for (i = 0; i < SIZEOF_ARRAY(iomix_funcs); i++) {
-			c += counters[i];
-			if (UNLIKELY(args->max_ops && c >= args->max_ops)) {
-				set_counter(args, c);
-				goto reap;
-			}
-		}
-	} while (keep_stressing(args));
+		pause();
+	} while (inc_counter_lock(args, counter_lock, false));
 
 	ret = EXIT_SUCCESS;
 reap:
-	set_counter(args, 0);
 	for (i = 0; i < SIZEOF_ARRAY(iomix_funcs); i++) {
-		add_counter(args, counters[i]);
-
 		if (pids[i]) {
 			(void)kill(pids[i], SIGALRM);
 			(void)kill(pids[i], SIGKILL);
@@ -914,9 +1058,9 @@ tidy:
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 	(void)close(fd);
 	(void)stress_temp_dir_rm_args(args);
-unmap:
+lock_destroy:
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
-	(void)munmap((void *)counters, sz);
+	(void)stress_lock_destroy(counter_lock);
 
 	return ret;
 }
@@ -930,5 +1074,6 @@ stressor_info_t stress_iomix_info = {
 	.stressor = stress_iomix,
 	.class = CLASS_FILESYSTEM | CLASS_OS,
 	.opt_set_funcs = opt_set_funcs,
+	.verify = VERIFY_ALWAYS,
 	.help = help
 };
