@@ -94,7 +94,6 @@ again:
 		return -1;
 	}
 	if (pid == 0) {
-		(void)setpgid(0, g_pgrp);
 		stress_parent_died_alarm();
 		(void)sched_settings_apply(true);
 
@@ -118,7 +117,6 @@ again:
 		} while (keep_stressing(args));
 		_exit(EXIT_SUCCESS);
 	}
-	(void)setpgid(pid, g_pgrp);
 	return pid;
 }
 
@@ -138,7 +136,7 @@ static int stress_try_lease(
 	if (fd < 0) {
 		int ret;
 
-		ret = exit_status(errno);
+		ret = stress_exit_status(errno);
 		pr_err("%s: open failed (parent): errno=%d: (%s)\n",
 			args->name, errno, strerror(errno));
 		return ret;
@@ -201,13 +199,13 @@ static int stress_lease(const stress_args_t *args)
 
 	ret = stress_temp_dir_mk_args(args);
 	if (ret < 0)
-		return exit_status(-ret);
+		return stress_exit_status(-ret);
 	(void)stress_temp_filename_args(args,
 		filename, sizeof(filename), stress_mwc32());
 
 	fd = creat(filename, S_IRUSR | S_IWUSR);
 	if (fd < 0) {
-		ret = exit_status(errno);
+		ret = stress_exit_status(errno);
 		pr_err("%s: creat failed: errno=%d: (%s)\n",
 			args->name, errno, strerror(errno));
 		(void)stress_temp_dir_rm_args(args);
