@@ -103,14 +103,8 @@ static hint void *name(void *dest, const void *src, size_t n)		\
 TEST_NAIVE_MEMCPY(test_naive_memcpy, NOINLINE)
 TEST_NAIVE_MEMCPY(test_naive_memcpy_o0, NOINLINE OPTIMIZE0)
 TEST_NAIVE_MEMCPY(test_naive_memcpy_o1, NOINLINE OPTIMIZE1)
-#if NEED_CLANG(9, 0, 0)
-TEST_NAIVE_MEMCPY(test_naive_memcpy_o2, NOINLINE OPTIMIZE2 TARGET_CLONES)
-TEST_NAIVE_MEMCPY(test_naive_memcpy_o3, NOINLINE OPTIMIZE3 TARGET_CLONES)
-#else
-/* older gcc versions segfault on TARGET_CLONES */
 TEST_NAIVE_MEMCPY(test_naive_memcpy_o2, NOINLINE OPTIMIZE2)
 TEST_NAIVE_MEMCPY(test_naive_memcpy_o3, NOINLINE OPTIMIZE3)
-#endif
 
 #define TEST_NAIVE_MEMMOVE(name, hint)					\
 static hint void *name(void *dest, const void *src, size_t n)		\
@@ -135,14 +129,8 @@ static hint void *name(void *dest, const void *src, size_t n)		\
 TEST_NAIVE_MEMMOVE(test_naive_memmove, NOINLINE)
 TEST_NAIVE_MEMMOVE(test_naive_memmove_o0, NOINLINE OPTIMIZE0)
 TEST_NAIVE_MEMMOVE(test_naive_memmove_o1, NOINLINE OPTIMIZE1)
-#if NEED_CLANG(9, 0, 0)
-TEST_NAIVE_MEMMOVE(test_naive_memmove_o2, NOINLINE OPTIMIZE2 TARGET_CLONES)
-TEST_NAIVE_MEMMOVE(test_naive_memmove_o3, NOINLINE OPTIMIZE3 TARGET_CLONES)
-#else
-/* older gcc versions segfault on TARGET_CLONES */
 TEST_NAIVE_MEMMOVE(test_naive_memmove_o2, NOINLINE OPTIMIZE2)
 TEST_NAIVE_MEMMOVE(test_naive_memmove_o3, NOINLINE OPTIMIZE3)
-#endif
 
 static NOINLINE void stress_memcpy_libc(
 	stress_buffer_t *b,
