@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,14 +16,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
+#include "core-capabilities.h"
+
+#if defined(HAVE_LINUX_MEMPOLICY_H)
+#include <linux/mempolicy.h>
+#endif
 
 static const stress_help_t help[] = {
 	{ NULL,	"numa N",	"start N workers stressing NUMA interfaces" },
@@ -118,7 +118,7 @@ static void stress_numa_free_nodes(stress_node_t *nodes)
  *  hex_to_int()
  *	convert ASCII hex digit to integer
  */
-static inline int hex_to_int(const char ch)
+static int hex_to_int(const char ch)
 {
 	if ((ch >= '0') && (ch <= '9'))
 		return ch - '0';
@@ -205,6 +205,13 @@ static long stress_numa_get_mem_nodes(
 	return n;
 }
 
+static void stress_numa_array_zero(void *array, size_t nmemb, size_t size)
+{
+	const size_t n = nmemb * size;
+
+	(void)memset(array, 0, n);
+}
+
 /*
  *  stress_numa()
  *	stress the Linux NUMA interfaces
@@ -213,13 +220,13 @@ static int stress_numa(const stress_args_t *args)
 {
 	long numa_nodes;
 	unsigned long max_nodes;
-	const unsigned long lbits = NUMA_LONG_BITS;
 	const unsigned long page_size = args->page_size;
 	const unsigned long num_pages = MMAP_SZ / page_size;
 	uint8_t *buf;
 	stress_node_t *n;
 	int rc = EXIT_FAILURE;
 	const bool cap_sys_nice = stress_check_capability(SHIM_CAP_SYS_NICE);
+	int *status, *dest_nodes;
 
 	numa_nodes = stress_numa_get_mem_nodes(&n, &max_nodes);
 	if (numa_nodes < 1) {
@@ -233,24 +240,41 @@ static int stress_numa(const stress_args_t *args)
 			args->name, numa_nodes, max_nodes);
 	}
 
+	status = (int *)calloc(num_pages, sizeof(*status));
+	if (!status) {
+		pr_inf("%s: cannot allocate status array of %lu elements, skipping stressor\n",
+			args->name, num_pages);
+		rc = EXIT_NO_RESOURCE;
+		goto numa_free;
+	}
+	dest_nodes = (int *)calloc(num_pages, sizeof(*dest_nodes));
+	if (!dest_nodes) {
+		pr_inf("%s: cannot allocate dest_nodes array of %lu elements, skipping stressor\n",
+			args->name, num_pages);
+		rc = EXIT_NO_RESOURCE;
+		goto status_free;
+	}
+
 	/*
 	 *  We need a buffer to migrate around NUMA nodes
 	 */
 	buf = mmap(NULL, MMAP_SZ, PROT_READ | PROT_WRITE,
 		MAP_ANONYMOUS | MAP_PRIVATE, 0, 0);
 	if (buf == MAP_FAILED) {
-		rc = exit_status(errno);
+		rc = stress_exit_status(errno);
 		pr_fail("%s: mmap'd region of %zu bytes failed\n",
 			args->name, (size_t)MMAP_SZ);
-		goto numa_free;
+		goto dest_nodes_free;
 	}
 
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
 	do {
-		int j, mode, ret, status[num_pages], dest_nodes[num_pages];
+		int j, mode, ret;
 		long lret;
-		unsigned long i, node_mask[lbits], old_node_mask[lbits];
+		unsigned long i;
+		unsigned long node_mask[NUMA_LONG_BITS];
+		unsigned long old_node_mask[NUMA_LONG_BITS];
 		unsigned long max_node_id_count;
 		void *pages[num_pages];
 		uint8_t *ptr;
@@ -258,6 +282,8 @@ static int stress_numa(const stress_args_t *args)
 		unsigned cpu, curr_node;
 		struct shim_getcpu_cache cache;
 
+		stress_numa_array_zero(node_mask, 1, sizeof(node_mask));
+
 		/*
 		 *  Fetch memory policy
 		 */
@@ -272,31 +298,20 @@ static int stress_numa(const stress_args_t *args)
 		}
 
 		/* Exercise invalid max_nodes */
-		ret = shim_get_mempolicy(&mode, node_mask, 0,
-					 buf, MPOL_F_NODE);
-		(void)ret;
+		VOID_RET(int, shim_get_mempolicy(&mode, node_mask, 0, buf, MPOL_F_NODE));
 
 		/* Exercise invalid flag */
-		ret = shim_get_mempolicy(&mode, node_mask, max_nodes, buf, ~0UL);
-		(void)ret;
+		VOID_RET(int, shim_get_mempolicy(&mode, node_mask, max_nodes, buf, ~0UL));
 
 		/* Exercise invalid NULL addr condition */
-		ret = shim_get_mempolicy(&mode, node_mask, max_nodes,
-					 NULL, MPOL_F_ADDR);
-		(void)ret;
+		VOID_RET(int, shim_get_mempolicy(&mode, node_mask, max_nodes, NULL, MPOL_F_ADDR));
 
-		ret = shim_get_mempolicy(&mode, node_mask, max_nodes,
-					 buf, MPOL_F_NODE);
-		(void)ret;
+		VOID_RET(int, shim_get_mempolicy(&mode, node_mask, max_nodes, buf, MPOL_F_NODE));
 
 		/* Exercise MPOL_F_MEMS_ALLOWED flag syscalls */
-		ret = shim_get_mempolicy(&mode, node_mask, max_nodes,
-					 buf, MPOL_F_MEMS_ALLOWED);
-		(void)ret;
+		VOID_RET(int, shim_get_mempolicy(&mode, node_mask, max_nodes, buf, MPOL_F_MEMS_ALLOWED));
 
-		ret = shim_get_mempolicy(&mode, node_mask, max_nodes,
-					 buf, MPOL_F_MEMS_ALLOWED | MPOL_F_NODE);
-		(void)ret;
+		VOID_RET(int, shim_get_mempolicy(&mode, node_mask, max_nodes, buf, MPOL_F_MEMS_ALLOWED | MPOL_F_NODE));
 
 		if (!keep_stressing_flag())
 			break;
@@ -325,53 +340,58 @@ static int stress_numa(const stress_args_t *args)
 			mode |= MPOL_F_RELATIVE_NODES;
 #endif
 
-		switch (stress_mwc8() % 10) {
+		switch (stress_mwc8() % 11) {
 		case 0:
 #if defined(MPOL_DEFAULT)
-			ret = shim_set_mempolicy(MPOL_DEFAULT | mode, NULL, max_nodes);
+			VOID_RET(long, shim_set_mempolicy(MPOL_DEFAULT | mode, NULL, max_nodes));
 			break;
 #endif
 		case 1:
 #if defined(MPOL_BIND)
-			ret = shim_set_mempolicy(MPOL_BIND | mode, node_mask, max_nodes);
+			VOID_RET(long, shim_set_mempolicy(MPOL_BIND | mode, node_mask, max_nodes));
 			break;
 #endif
 		case 2:
 #if defined(MPOL_INTERLEAVE)
-			ret = shim_set_mempolicy(MPOL_INTERLEAVE | mode, node_mask, max_nodes);
+			VOID_RET(long, shim_set_mempolicy(MPOL_INTERLEAVE | mode, node_mask, max_nodes));
 			break;
 #endif
 		case 3:
 #if defined(MPOL_PREFERRED)
-			ret = shim_set_mempolicy(MPOL_PREFERRED | mode, node_mask, max_nodes);
+			VOID_RET(long, shim_set_mempolicy(MPOL_PREFERRED | mode, node_mask, max_nodes));
 			break;
 #endif
 		case 4:
 #if defined(MPOL_LOCAL)
-			ret = shim_set_mempolicy(MPOL_LOCAL | mode, node_mask, max_nodes);
+			VOID_RET(long, shim_set_mempolicy(MPOL_LOCAL | mode, node_mask, max_nodes));
 			break;
 #endif
 		case 5:
 #if defined(MPOL_PREFERRED_MANY)
-			ret = shim_set_mempolicy(MPOL_PREFERRED_MANY | mode, node_mask, max_nodes);
+			VOID_RET(long, shim_set_mempolicy(MPOL_PREFERRED_MANY | mode, node_mask, max_nodes));
 			break;
 #endif
 		case 6:
-			ret = shim_set_mempolicy(0, node_mask, max_nodes);
+			VOID_RET(long, shim_set_mempolicy(0, node_mask, max_nodes));
 			break;
 		case 7:
-			ret = shim_set_mempolicy(mode, node_mask, max_nodes);
+			VOID_RET(long, shim_set_mempolicy(mode, node_mask, max_nodes));
 			break;
 		case 8:
 			/* Invalid mode */
-			ret = shim_set_mempolicy(mode | MPOL_F_STATIC_NODES | MPOL_F_RELATIVE_NODES, node_mask, max_nodes);
+			VOID_RET(long, shim_set_mempolicy(mode | MPOL_F_STATIC_NODES | MPOL_F_RELATIVE_NODES, node_mask, max_nodes));
 			break;
+#if defined(MPOL_F_NUMA_BALANCING) &&	\
+    defined(MPOL_LOCAL)
+		case 9:
+			/* Invalid  MPOL_F_NUMA_BALANCING | MPOL_LOCAL */
+			VOID_RET(long, shim_set_mempolicy(MPOL_F_NUMA_BALANCING | MPOL_LOCAL, node_mask, max_nodes));
+			break;
+#endif
 		default:
 			/* Intentionally invalid mode */
-			ret = shim_set_mempolicy(~0, node_mask, max_nodes);
+			VOID_RET(long, shim_set_mempolicy(~0, node_mask, max_nodes));
 		}
-		(void)ret;
-
 
 		/*
 		 *  Fetch CPU and node, we just waste some cycled
@@ -380,7 +400,7 @@ static int stress_numa(const stress_args_t *args)
 		(void)shim_getcpu(&cpu, &curr_node, NULL);
 
 		/* Initialised cache to be safe */
-		(void)memset(&cache, 0, sizeof(cache));
+		stress_numa_array_zero(&cache, 1, sizeof(cache));
 
 		/*
 		 * tcache argument is unused in getcpu currently.
@@ -394,7 +414,7 @@ static int stress_numa(const stress_args_t *args)
 		 *  mbind the buffer, first try MPOL_STRICT which
 		 *  may fail with EIO
 		 */
-		(void)memset(node_mask, 0, sizeof(node_mask));
+		stress_numa_array_zero(node_mask, 1, sizeof(node_mask));
 		STRESS_SETBIT(node_mask, n->node_id);
 		lret = shim_mbind((void *)buf, MMAP_SZ, MPOL_BIND, node_mask,
 			max_nodes, MPOL_MF_STRICT);
@@ -413,7 +433,7 @@ static int stress_numa(const stress_args_t *args)
 		/*
 		 *  mbind the buffer, now try MPOL_DEFAULT
 		 */
-		(void)memset(node_mask, 0, sizeof(node_mask));
+		stress_numa_array_zero(node_mask, 1, sizeof(node_mask));
 		STRESS_SETBIT(node_mask, n->node_id);
 		lret = shim_mbind((void *)buf, MMAP_SZ, MPOL_BIND, node_mask,
 			max_nodes, MPOL_DEFAULT);
@@ -430,37 +450,30 @@ static int stress_numa(const stress_args_t *args)
 			break;
 
 		/* Exercise invalid start address */
-		lret = shim_mbind((void *)(buf + 7), MMAP_SZ, MPOL_BIND, node_mask,
-			max_nodes, MPOL_MF_STRICT);
-		(void)lret;
+		VOID_RET(long, shim_mbind((void *)(buf + 7), MMAP_SZ, MPOL_BIND, node_mask,
+			max_nodes, MPOL_MF_STRICT));
 
 		/* Exercise wrap around */
-		lret = shim_mbind((void *)(~(uintptr_t)0 & ~(page_size - 1)), page_size * 2,
-			MPOL_BIND, node_mask, max_nodes, MPOL_MF_STRICT);
-		(void)lret;
+		VOID_RET(long, shim_mbind((void *)(~(uintptr_t)0 & ~(page_size - 1)), page_size * 2,
+			MPOL_BIND, node_mask, max_nodes, MPOL_MF_STRICT));
 
 		/* Exercise invalid length */
-		lret = shim_mbind((void *)buf, ~0UL, MPOL_BIND, node_mask,
-			max_nodes, MPOL_MF_STRICT);
-		(void)lret;
+		VOID_RET(long, shim_mbind((void *)buf, ~0UL, MPOL_BIND, node_mask,
+			max_nodes, MPOL_MF_STRICT));
 
 		/* Exercise zero length, allowed, but is a no-op */
-		lret = shim_mbind((void *)buf, 0, MPOL_BIND, node_mask,
-			max_nodes, MPOL_MF_STRICT);
-		(void)lret;
+		VOID_RET(long, shim_mbind((void *)buf, 0, MPOL_BIND, node_mask,
+			max_nodes, MPOL_MF_STRICT));
 
 		/* Exercise invalid max_nodes */
-		lret = shim_mbind((void *)buf, MMAP_SZ, MPOL_BIND, node_mask,
-			0, MPOL_MF_STRICT);
-		(void)lret;
-		lret = shim_mbind((void *)buf, MMAP_SZ, MPOL_BIND, node_mask,
-			0xffffffff, MPOL_MF_STRICT);
-		(void)lret;
+		VOID_RET(long, shim_mbind((void *)buf, MMAP_SZ, MPOL_BIND, node_mask,
+			0, MPOL_MF_STRICT));
+		VOID_RET(long, shim_mbind((void *)buf, MMAP_SZ, MPOL_BIND, node_mask,
+			0xffffffff, MPOL_MF_STRICT));
 
 		/* Exercise invalid flags */
-		lret = shim_mbind((void *)buf, MMAP_SZ, MPOL_BIND, node_mask,
-			max_nodes, ~0U);
-		(void)lret;
+		VOID_RET(long, shim_mbind((void *)buf, MMAP_SZ, MPOL_BIND, node_mask,
+			max_nodes, ~0U));
 
 		/* Check mbind syscall cannot succeed without capability */
 		if (!cap_sys_nice) {
@@ -478,40 +491,33 @@ static int stress_numa(const stress_args_t *args)
 		/*
 		 *  Migrate all this processes pages to the current new node
 		 */
-		(void)memset(old_node_mask, 0, sizeof(old_node_mask));
+		stress_numa_array_zero(old_node_mask, 1, sizeof(old_node_mask));
 		max_node_id_count = max_nodes;
-		for (i = 0; max_node_id_count >= lbits && i < lbits; i++) {
+		for (i = 0; (max_node_id_count >= NUMA_LONG_BITS) && (i < NUMA_LONG_BITS); i++) {
 			old_node_mask[i] = ULONG_MAX;
-			max_node_id_count -= lbits;
+			max_node_id_count -= NUMA_LONG_BITS;
 		}
-		if (i < lbits)
+		if (i < NUMA_LONG_BITS)
 			old_node_mask[i] = (1UL << max_node_id_count) - 1;
-		(void)memset(node_mask, 0, sizeof(node_mask));
+		stress_numa_array_zero(node_mask, 1, sizeof(node_mask));
 		STRESS_SETBIT(node_mask, n->node_id);
 
 		/*
 	 	 *  Ignore any failures, this is not strictly important
 		 */
-		lret = shim_migrate_pages(args->pid, max_nodes,
-			old_node_mask, node_mask);
-		(void)lret;
+		VOID_RET(long, shim_migrate_pages(args->pid, max_nodes,
+			old_node_mask, node_mask));
 
 		/*
 		 *  Exercise illegal pid
 		 */
-		lret = shim_migrate_pages(~0, max_nodes, old_node_mask,
-			node_mask);
-		(void)lret;
+		VOID_RET(long, shim_migrate_pages(~0, max_nodes, old_node_mask, node_mask));
 
 		/*
 		 *  Exercise illegal max_nodes
 		 */
-		lret = shim_migrate_pages(args->pid, ~0UL, old_node_mask,
-			node_mask);
-		(void)lret;
-		lret = shim_migrate_pages(args->pid, 0, old_node_mask,
-			node_mask);
-		(void)lret;
+		VOID_RET(long, shim_migrate_pages(args->pid, ~0UL, old_node_mask, node_mask));
+		VOID_RET(long, shim_migrate_pages(args->pid, 0, old_node_mask, node_mask));
 
 		if (!keep_stressing_flag())
 			break;
@@ -525,7 +531,7 @@ static int stress_numa(const stress_args_t *args)
 				pages[i] = ptr;
 				dest_nodes[i] = (int)n_tmp->node_id;
 			}
-			(void)memset(status, 0, sizeof(status));
+			stress_numa_array_zero(status, num_pages, sizeof(*status));
 			lret = shim_move_pages(args->pid, num_pages, pages,
 				dest_nodes, status, MPOL_MF_MOVE);
 			if (lret < 0) {
@@ -542,54 +548,46 @@ static int stress_numa(const stress_args_t *args)
 
 #if defined(MPOL_MF_MOVE_ALL)
 		/* Exercise MPOL_MF_MOVE_ALL, this needs privilege, ignore failure */
-		(void)memset(status, 0, sizeof(status));
+		stress_numa_array_zero(status, num_pages, sizeof(*status));
 		pages[0] = buf;
-		lret = shim_move_pages(args->pid, num_pages, pages, dest_nodes, status, MPOL_MF_MOVE_ALL);
-		(void)lret;
+		VOID_RET(long, shim_move_pages(args->pid, num_pages, pages, dest_nodes, status, MPOL_MF_MOVE_ALL));
 #endif
 
 		/* Exercise invalid pid on move pages */
-		(void)memset(status, 0, sizeof(status));
+		stress_numa_array_zero(status, num_pages, sizeof(*status));
 		pages[0] = buf;
-		lret = shim_move_pages(~0, 1, pages, dest_nodes, status, MPOL_MF_MOVE);
-		(void)lret;
+		VOID_RET(long, shim_move_pages(~0, 1, pages, dest_nodes, status, MPOL_MF_MOVE));
 
 		/* Exercise 0 nr_pages */
-		(void)memset(status, 0, sizeof(status));
+		stress_numa_array_zero(status, num_pages, sizeof(*status));
 		pages[0] = buf;
-		lret = shim_move_pages(args->pid, 0, pages, dest_nodes, status, MPOL_MF_MOVE);
-		(void)lret;
+		VOID_RET(long, shim_move_pages(args->pid, 0, pages, dest_nodes, status, MPOL_MF_MOVE));
 
 		/* Exercise invalid move flags */
-		(void)memset(status, 0, sizeof(status));
+		stress_numa_array_zero(status, num_pages, sizeof(*status));
 		pages[0] = buf;
-		lret = shim_move_pages(args->pid, 1, pages, dest_nodes, status, ~0);
-		(void)lret;
+		VOID_RET(long, shim_move_pages(args->pid, 1, pages, dest_nodes, status, ~0));
 
 		/* Exercise zero flag, should succeed */
-		(void)memset(status, 0, sizeof(status));
+		stress_numa_array_zero(status, num_pages, sizeof(*status));
 		pages[0] = buf;
-		lret = shim_move_pages(args->pid, 1, pages, dest_nodes, status, 0);
-		(void)lret;
+		VOID_RET(long, shim_move_pages(args->pid, 1, pages, dest_nodes, status, 0));
 
 		/* Exercise invalid address */
-		(void)memset(status, 0, sizeof(status));
+		stress_numa_array_zero(status, num_pages, sizeof(*status));
 		pages[0] = (void *)(~(uintptr_t)0 & ~(args->page_size - 1));
-		lret = shim_move_pages(args->pid, 1, pages, dest_nodes, status, MPOL_MF_MOVE);
-		(void)lret;
+		VOID_RET(long, shim_move_pages(args->pid, 1, pages, dest_nodes, status, MPOL_MF_MOVE));
 
 		/* Exercise invalid dest_node */
-		(void)memset(status, 0, sizeof(status));
+		stress_numa_array_zero(status, num_pages, sizeof(*status));
 		pages[0] = buf;
 		dest_nodes[0] = ~0;
-		lret = shim_move_pages(args->pid, 1, pages, dest_nodes, status, MPOL_MF_MOVE);
-		(void)lret;
+		VOID_RET(long, shim_move_pages(args->pid, 1, pages, dest_nodes, status, MPOL_MF_MOVE));
 
 		/* Exercise NULL nodes */
-		(void)memset(status, 0, sizeof(status));
+		stress_numa_array_zero(status, num_pages, sizeof(*status));
 		pages[0] = buf;
-		lret = shim_move_pages(args->pid, 1, pages, NULL, status, MPOL_MF_MOVE);
-		(void)lret;
+		VOID_RET(long, shim_move_pages(args->pid, 1, pages, NULL, status, MPOL_MF_MOVE));
 
 		inc_counter(args);
 	} while (keep_stressing(args));
@@ -598,6 +596,11 @@ static int stress_numa(const stress_args_t *args)
 err:
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 	(void)munmap(buf, MMAP_SZ);
+
+dest_nodes_free:
+	free(dest_nodes);
+status_free:
+	free(status);
 numa_free:
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 	stress_numa_free_nodes(n);
@@ -608,6 +611,7 @@ numa_free:
 stressor_info_t stress_numa_info = {
 	.stressor = stress_numa,
 	.class = CLASS_CPU | CLASS_MEMORY | CLASS_OS,
+	.verify = VERIFY_ALWAYS,
 	.help = help
 };
 #else
