@@ -103,6 +103,10 @@ static int stress_oom_pipe_child(const stress_args_t *args, void *ctxt)
 
 	for (i = 0; keep_stressing(args) && (i < max_pipes); i++) {
 		int *pfd = fds + (2 * i);
+
+		if ((g_opt_flags & OPT_FLAGS_OOM_AVOID) && stress_low_memory(page_size))
+			break;
+
 		if (pipe(pfd) < 0) {
 			pfd[0] = -1;
 			pfd[1] = -1;
@@ -135,6 +139,8 @@ static int stress_oom_pipe_child(const stress_args_t *args, void *ctxt)
 
 			if ((fd[0] < 0) || (fd[1] < 0))
 				continue;
+			if ((g_opt_flags & OPT_FLAGS_OOM_AVOID) && stress_low_memory(max_size * 2))
+				break;
 			if (fcntl(fd[0], F_SETPIPE_SZ, max_size) < 0)
 				max_size = page_size;
 			if (fcntl(fd[1], F_SETPIPE_SZ, max_size) < 0)
