@@ -1,5 +1,6 @@
 /*
- * Copyright (C) 2013-2017 Canonical, Ltd.
+ * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,44 +16,49 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
 #define PIPE_STOP	"PS!"
 
+static const stress_help_t help[] = {
+	{ "p N", "pipe N",		"start N workers exercising pipe I/O" },
+	{ NULL,	"pipe-ops N",		"stop after N pipe I/O bogo operations" },
+	{ NULL,	"pipe-data-size N",	"set pipe size of each pipe write to N bytes" },
+#if defined(F_SETPIPE_SZ)
+	{ NULL,	"pipe-size N",		"set pipe size to N bytes" },
+#endif
+	{ NULL,	NULL,			NULL }
+};
+
 #if defined(F_SETPIPE_SZ)
 /*
  *  stress_set_pipe_size()
  *	set pipe size in bytes
  */
-void stress_set_pipe_size(const char *opt)
+static int stress_set_pipe_size(const char *opt)
 {
 	size_t pipe_size;
 
-	pipe_size = (size_t)get_uint64_byte(opt);
-	check_range_bytes("pipe-size", pipe_size, 4, 1024 * 1024);
-	set_setting("pipe-size", TYPE_ID_SIZE_T, &pipe_size);
+	pipe_size = (size_t)stress_get_uint64_byte(opt);
+	stress_check_range_bytes("pipe-size", pipe_size, 4, 1024 * 1024);
+	return stress_set_setting("pipe-size", TYPE_ID_SIZE_T, &pipe_size);
 }
+
 #endif
 
 /*
  *  stress_set_pipe_size()
  *	set pipe data write size in bytes
  */
-void stress_set_pipe_data_size(const char *opt)
+static int stress_set_pipe_data_size(const char *opt)
 {
 	size_t pipe_data_size;
 
-	pipe_data_size = (size_t)get_uint64_byte(opt);
-	check_range_bytes("pipe-data-size", pipe_data_size,
-		4, stress_get_pagesize());
-	set_setting("pipe-data-size,", TYPE_ID_SIZE_T, &pipe_data_size);
+	pipe_data_size = (size_t)stress_get_uint64_byte(opt);
+	stress_check_range_bytes("pipe-data-size", pipe_data_size,
+		4, stress_get_page_size());
+	return stress_set_setting("pipe-data-size,", TYPE_ID_SIZE_T, &pipe_data_size);
 }
 
 /*
@@ -87,7 +93,7 @@ static inline int pipe_memchk(char *buf, char val, const size_t sz)
  *	see if we can change the pipe size
  */
 static void pipe_change_size(
-	const args_t *args,
+	const stress_args_t *args,
 	const int fd,
 	const size_t pipe_size)
 {
@@ -97,7 +103,8 @@ static void pipe_change_size(
 	if (!pipe_size)
 		return;
 
-#if !(defined(__linux__) && NEED_GLIBC(2,9,0))
+#if !(defined(HAVE_PIPE2) &&	\
+    defined(O_DIRECT))
 	if (pipe_size < args->page_size)
 		return;
 #endif
@@ -127,22 +134,42 @@ static void pipe_change_size(
  *  stress_pipe
  *	stress by heavy pipe I/O
  */
-int stress_pipe(const args_t *args)
+static int stress_pipe(const stress_args_t *args)
 {
 	pid_t pid;
 	int pipefds[2];
 	size_t pipe_data_size = 512;
+	char *buf;
 
-	(void)get_setting("pipe-data-size", &pipe_data_size);
+	(void)stress_get_setting("pipe-data-size", &pipe_data_size);
 
-#if defined(__linux__) && NEED_GLIBC(2,9,0)
+	buf = (char *)mmap(NULL, pipe_data_size, PROT_READ | PROT_WRITE,
+				MAP_ANONYMOUS | MAP_PRIVATE, -1, 0);
+	if (buf == MAP_FAILED) {
+		pr_err("%s: failed to allocate buffer\n", args->name);
+		return EXIT_NO_RESOURCE;
+	}
+
+	(void)memset(pipefds, 0, sizeof(pipefds));
+
+#if defined(HAVE_PIPE2) &&	\
+    defined(O_DIRECT)
 	if (pipe2(pipefds, O_DIRECT) < 0) {
-		pr_fail_dbg("pipe2");
-		return EXIT_FAILURE;
+		/*
+		 *  Failed, fall back to standard pipe
+		 */
+		if (pipe(pipefds) < 0) {
+			pr_fail("%s: pipe failed, errno=%d (%s)\n",
+				args->name, errno, strerror(errno));
+			(void)munmap((void *)buf, pipe_data_size);
+			return EXIT_FAILURE;
+		}
 	}
 #else
 	if (pipe(pipefds) < 0) {
-		pr_fail_dbg("pipe");
+		pr_fail("%s: pipe failed, errno=%d (%s)\n",
+			args->name, errno, strerror(errno));
+		(void)munmap((void *)buf, pipe_data_size);
 		return EXIT_FAILURE;
 	}
 #endif
@@ -151,30 +178,42 @@ int stress_pipe(const args_t *args)
 	{
 		size_t pipe_size = 0;
 
-		(void)get_setting("pipe-size", &pipe_size);
+		(void)stress_get_setting("pipe-size", &pipe_size);
 		pipe_change_size(args, pipefds[0], pipe_size);
 		pipe_change_size(args, pipefds[1], pipe_size);
 	}
+#else
+	UNEXPECTED
 #endif
 
+	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 again:
 	pid = fork();
 	if (pid < 0) {
-		if (g_keep_stressing_flag && (errno == EAGAIN))
+		if (stress_redo_fork(errno))
 			goto again;
 		(void)close(pipefds[0]);
 		(void)close(pipefds[1]);
-		pr_fail_dbg("fork");
+		(void)munmap((void *)buf, pipe_data_size);
+		if (!keep_stressing(args))
+			goto finish;
+		pr_fail("%s: fork failed, errno=%d (%s)\n",
+			args->name, errno, strerror(errno));
 		return EXIT_FAILURE;
 	} else if (pid == 0) {
 		int val = 0;
+#if defined(FIONREAD)
+		int i = 0;
+#endif
+		const pid_t my_pid = getpid();
 
-		(void)setpgid(0, g_pgrp);
 		stress_parent_died_alarm();
+		(void)sched_settings_apply(true);
+
+		(void)stress_read_fdinfo(my_pid, pipefds[0]);
 
 		(void)close(pipefds[1]);
-		while (g_keep_stressing_flag) {
-			char buf[pipe_data_size];
+		while (keep_stressing_flag()) {
 			ssize_t n;
 
 			n = read(pipefds[0], buf, pipe_data_size);
@@ -182,55 +221,87 @@ again:
 				if ((errno == EAGAIN) || (errno == EINTR))
 					continue;
 				if (errno) {
-					pr_fail_dbg("read");
+					pr_fail("%s: read failed, errno=%d (%s)\n",
+						args->name, errno, strerror(errno));
 					break;
 				}
-				pr_fail_dbg("zero byte read");
+				pr_fail("%s: zero bytes read\n", args->name);
 				break;
 			}
-			if (!strcmp(buf, PIPE_STOP))
+
+#if defined(FIONREAD)
+			/* Occasionally exercise FIONREAD on read end */
+			if ((i++ & 0x1ff) == 0) {
+				int bytes;
+
+				VOID_RET(int, ioctl(pipefds[0], FIONREAD, &bytes));
+			}
+#endif
+			if (!strncmp(buf, PIPE_STOP, 3))
 				break;
 			if ((g_opt_flags & OPT_FLAGS_VERIFY) &&
-			    pipe_memchk(buf, val++, (size_t)n)) {
+			    pipe_memchk(buf, (char)val++, (size_t)n)) {
 				pr_fail("%s: pipe read error detected, "
 					"failed to read expected data\n", args->name);
 			}
 		}
 		(void)close(pipefds[0]);
-		exit(EXIT_SUCCESS);
+		(void)munmap((void *)buf, pipe_data_size);
+		_exit(EXIT_SUCCESS);
 	} else {
-		char buf[pipe_data_size];
 		int val = 0, status;
 
 		/* Parent */
-		(void)setpgid(pid, g_pgrp);
 		(void)close(pipefds[0]);
 
 		do {
 			ssize_t ret;
 
-			pipe_memset(buf, val++, pipe_data_size);
+			pipe_memset(buf, (char)val++, pipe_data_size);
 			ret = write(pipefds[1], buf, pipe_data_size);
 			if (ret <= 0) {
 				if ((errno == EAGAIN) || (errno == EINTR))
 					continue;
 				if (errno) {
-					pr_fail_dbg("write");
+					pr_fail("%s: write failed, errno=%d (%s)\n",
+						args->name, errno, strerror(errno));
 					break;
 				}
 				continue;
 			}
 			inc_counter(args);
-		} while (keep_stressing());
+		} while (keep_stressing(args));
 
-		(void)strncpy(buf, PIPE_STOP, pipe_data_size);
-		if (write(pipefds[1], buf, sizeof(buf)) <= 0) {
+		(void)memset(buf, 0, pipe_data_size);
+		(void)memcpy(buf, PIPE_STOP, sizeof(PIPE_STOP));
+		if (write(pipefds[1], buf, pipe_data_size) <= 0) {
 			if (errno != EPIPE)
-				pr_fail_dbg("termination write");
+				pr_fail("%s: termination write failed, errno=%d (%s)\n",
+					args->name, errno, strerror(errno));
 		}
 		(void)kill(pid, SIGKILL);
-		(void)waitpid(pid, &status, 0);
+		(void)shim_waitpid(pid, &status, 0);
 		(void)close(pipefds[1]);
+		(void)munmap((void *)buf, pipe_data_size);
 	}
+finish:
+	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
+
 	return EXIT_SUCCESS;
 }
+
+static const stress_opt_set_func_t opt_set_funcs[] = {
+#if defined(F_SETPIPE_SZ)
+	{ OPT_pipe_size,	stress_set_pipe_size },
+#endif
+	{ OPT_pipe_data_size,	stress_set_pipe_data_size },
+	{ 0,			NULL }
+};
+
+stressor_info_t stress_pipe_info = {
+	.stressor = stress_pipe,
+	.class = CLASS_PIPE_IO | CLASS_MEMORY | CLASS_OS,
+	.opt_set_funcs = opt_set_funcs,
+	.verify = VERIFY_OPTIONAL,
+	.help = help
+};
