@@ -18,6 +18,7 @@
  *
  */
 #include "stress-ng.h"
+#include "core-pthread.h"
 #include "core-put.h"
 
 #if defined(HAVE_ASM_MTRR_H)
@@ -245,11 +246,21 @@ static inline void stress_proc_rw(
 		}
 #endif
 
+		/*
+		 *  Random sized reads at non-power of two integers
+		 *  on /proc/bus/pci/00 on SH4 5.16 kernels trips
+		 *  SIGBUS/SIGSEGV faults. Currenly skip this test.
+		 */
+#if defined(STRESS_ARCH_SH4)
+		if (!strncmp(path, "/proc/bus/pci/00", 16))
+			goto next;
+#endif
 		/*
 		 *  Multiple randomly sized reads
 		 */
 		for (i = 0; i < 4096 * PROC_BUF_SZ; i++) {
-			ssize_t sz = 1 + (stress_mwc32() % sizeof(buffer));
+			const ssize_t sz = 1 + (stress_mwc32() % sizeof(buffer));
+
 			if (!keep_stressing_flag())
 				break;
 			ret = read(fd, buffer, (size_t)sz);
@@ -423,6 +434,10 @@ mmap_test:
                         (void)sigemptyset(&sigmask);
                         (void)sigaddset(&sigmask, SIGPIPE);
 
+			fds[0].fd = fd;
+			fds[0].events = POLLIN;
+			fds[0].revents = 0;
+
 			VOID_RET(int, ppoll(fds, 1, &ts, &sigmask));
 		}
 #endif
@@ -630,6 +645,7 @@ static char *stress_random_pid(void)
 	 */
 	for (i = 0, j = 0; i < 32; i++) {
 		char *name;
+
 		j += (size_t)stress_mwc32();
 		j %= (size_t)n;
 
