@@ -169,11 +169,10 @@ static int stress_shm_sysv_check(
  */
 static void exercise_shmat(
 	const int shm_id,
-	const size_t page_size,
-	const size_t sz)
+	const size_t sz,
+	uint64_t *buffer)
 {
 	void *addr;
-	uint64_t buffer[(page_size / sizeof(uint64_t)) + 1];
 	/* Unaligned buffer */
 	const uint8_t *unaligned = ((uint8_t *)buffer) + 1;
 
@@ -263,7 +262,8 @@ static void exercise_shmat(
 	 * Exercise valid shmat syscall with unaligned
 	 * page address but specifying SHM_RND flag
 	 */
-#if defined(SHM_RND)
+#if defined(SHM_RND) &&	\
+    !defined(__FreeBSD__)
 	addr = shmat(shm_id, unaligned, SHM_RND);
 	if (addr != (void *) -1)
 		(void)shmdt(addr);
@@ -553,9 +553,20 @@ static int stress_shm_sysv_child(
 	int mask = ~0;
 	uint32_t instances = args->num_instances;
 	const bool cap_ipc_lock = stress_check_capability(SHIM_CAP_IPC_LOCK);
+	const size_t buffer_size = (page_size / sizeof(uint64_t)) + 1;
+	uint64_t *buffer;
 
-	if (stress_sig_stop_stressing(args->name, SIGALRM) < 0)
+	buffer = calloc(buffer_size, sizeof(*buffer));
+	if (!buffer) {
+		pr_inf("%s: cannot allocate %zu sized buffer, skipping stressor\n",
+			args->name, buffer_size);
+		return EXIT_NO_RESOURCE;
+	}
+
+	if (stress_sig_stop_stressing(args->name, SIGALRM) < 0) {
+		free(buffer);
 		return EXIT_FAILURE;
+	}
 
 	(void)memset(addrs, 0, sizeof(addrs));
 	(void)memset(keys, 0, sizeof(keys));
@@ -576,10 +587,10 @@ static int stress_shm_sysv_child(
 			int shm_id = -1, count = 0;
 			void *addr;
 			key_t key = 0;
-			size_t shmall, freemem, totalmem, freeswap;
+			size_t shmall, freemem, totalmem, freeswap, totalswap;
 
 			/* Try hard not to overcommit at this current time */
-			stress_get_memlimits(&shmall, &freemem, &totalmem, &freeswap);
+			stress_get_memlimits(&shmall, &freemem, &totalmem, &freeswap, &totalswap);
 			shmall /= instances;
 			freemem /= instances;
 			if ((shmall > page_size) && sz > shmall)
@@ -618,7 +629,6 @@ static int stress_shm_sysv_child(
 						goto reap;
 				} while (!unique);
 
-errno = 0;
 				shm_id = shmget(key, sz,
 					IPC_CREAT | IPC_EXCL |
 					S_IRUSR | S_IWUSR | rnd_flag);
@@ -656,13 +666,11 @@ errno = 0;
 				goto reap;
 			}
 
-			exercise_shmat(shm_id, args->page_size, sz);
-
 			addr = shmat(shm_id, NULL, 0);
 			if (addr == (char *) -1) {
 				ok = false;
-				pr_fail("%s: shmat failed, errno=%d (%s)\n",
-					args->name, errno, strerror(errno));
+				pr_fail("%s: shmat failed on id %d, errno=%d (%s)\n",
+					args->name, shm_id, errno, strerror(errno));
 				rc = EXIT_FAILURE;
 				goto reap;
 			}
@@ -777,6 +785,7 @@ errno = 0;
 #if defined(__linux__)
 			stress_shm_sysv_linux_proc_map(addr, sz);
 #endif
+			exercise_shmat(shm_id, sz, buffer);
 			inc_counter(args);
 		}
 
@@ -853,6 +862,7 @@ reap:
 			args->name, errno, strerror(errno));
 		rc = EXIT_FAILURE;
 	}
+	free(buffer);
 
 	return rc;
 }
@@ -923,7 +933,6 @@ fork_again:
 			bool get_procinfo = true;
 #endif
 
-			(void)setpgid(pid, g_pgrp);
 			stress_set_oom_adjustment(args->name, false);
 			(void)close(pipefds[1]);
 
@@ -975,6 +984,8 @@ fork_again:
 					restarts++;
 				}
 			}
+			if (WIFEXITED(status))
+				rc = WEXITSTATUS(status);
 			(void)close(pipefds[0]);
 			/*
 			 *  The child may have been killed by the OOM killer or
@@ -989,7 +1000,6 @@ fork_again:
 			}
 		} else {
 			/* Child, stress memory */
-			(void)setpgid(0, g_pgrp);
 			stress_parent_died_alarm();
 			(void)sched_settings_apply(true);
 
