@@ -216,7 +216,6 @@ static void stress_swap_check_swapped(
 {
 	unsigned char *vec;
 	register size_t n = 0;
-	register uint32_t i;
 
 	*swapped_total += npages;
 
@@ -224,9 +223,12 @@ static void stress_swap_check_swapped(
 	if (!vec)
 		return;
 
-	shim_mincore(addr, page_size * (size_t)npages, vec);
-	for (i = 0; i < npages; i++)
-		n += ((vec[i] & 1) == 0);
+	if (shim_mincore(addr, page_size * (size_t)npages, vec) == 0) {
+		register uint32_t i;
+
+		for (i = 0; i < npages; i++)
+			n += ((vec[i] & 1) == 0);
+	}
 
 	*swapped_out += n;
 	free(vec);
@@ -256,7 +258,7 @@ static int stress_swap(const stress_args_t *args)
 
 	ret = stress_temp_dir_mk_args(args);
 	if (ret < 0) {
-		ret = exit_status(-ret);
+		ret = stress_exit_status(-ret);
 		goto tidy_free;
 	}
 
@@ -264,7 +266,7 @@ static int stress_swap(const stress_args_t *args)
 		filename, sizeof(filename), stress_mwc32());
 	fd = open(filename, O_CREAT | O_RDWR, S_IRUSR);
 	if (fd < 0) {
-		ret = exit_status(errno);
+		ret = stress_exit_status(errno);
 		pr_fail("%s: open swap file %s failed, errno=%d (%s)\n",
 			args->name, filename, errno, strerror(errno));
 		goto tidy_rm;
