@@ -369,7 +369,7 @@ static int bad_getcwd(void *addr)
 #if defined(HAVE_GETDOMAINNAME)
 static int bad_getdomainname(void *addr)
 {
-	return getdomainname((char *)addr, 8192);
+	return shim_getdomainname((char *)addr, 8192);
 }
 #endif
 
@@ -1055,7 +1055,6 @@ static inline int stress_do_syscall(
 				_exit(EXIT_FAILURE);
 		}
 
-		(void)setpgid(0, g_pgrp);
 		stress_parent_died_alarm();
 		(void)sched_settings_apply(true);
 
