@@ -90,10 +90,7 @@ static int stress_set_timerfd_freq(const char *opt)
 
 static int stress_set_timerfd_rand(const char *opt)
 {
-	bool timerfd_rand = true;
-
-	(void)opt;
-	return stress_set_setting("timerfd-rand", TYPE_ID_BOOL, &timerfd_rand);
+	return stress_set_setting_true("timerfd-rand", opt);
 }
 
 static const stress_opt_set_func_t opt_set_funcs[] = {
@@ -123,7 +120,7 @@ static void stress_timerfd_set(
 
 	if (timerfd_rand) {
 		/* Mix in some random variation */
-		double r = ((double)(stress_mwc32() % 10000) - 5000.0) / 40000.0;
+		const double r = ((double)(stress_mwc32() % 10000) - 5000.0) / 40000.0;
 		rate = rate_ns + (rate_ns * r);
 	} else {
 		rate = rate_ns;
@@ -221,14 +218,14 @@ static int stress_timerfd(const stress_args_t *args)
 	/* Create a non valid timerfd file descriptor */
 	ret = stress_temp_dir_mk_args(args);
 	if (ret < 0) {
-		rc = exit_status(-ret);
+		rc = stress_exit_status(-ret);
 		goto dir_rm;
 	}
 	(void)stress_temp_filename_args(args, file_fd_name, sizeof(file_fd_name), stress_mwc32());
 	file_fd = open(file_fd_name, O_CREAT | O_RDWR, S_IRUSR | S_IWUSR);
 	if (file_fd < 0) {
 		pr_err("%s: cannot create %s\n", args->name, file_fd_name);
-		rc = exit_status(errno);
+		rc = stress_exit_status(errno);
 		goto close_file_fd;
 	}
 	(void)shim_unlink(file_fd_name);
