@@ -86,10 +86,7 @@ static int stress_tun_supported(const char *name)
 
 static int stress_set_tun_tap(const char *opt)
 {
-	bool tun_tap = true;
-
-	(void)opt;
-	return stress_set_setting("tun-tap", TYPE_ID_BOOL, &tun_tap);
+	return stress_set_setting_true("tun-tap", opt);
 }
 
 /*
@@ -187,7 +184,6 @@ static int stress_tun(const stress_args_t *args)
 			ssize_t n;
 			char buffer[4];
 
-			(void)setpgid(0, g_pgrp);
 			stress_parent_died_alarm();
 			(void)sched_settings_apply(true);
 
