@@ -190,7 +190,7 @@ static int stress_set_zombie_max(const char *opt)
 	uint32_t zombie_max;
 
 	zombie_max = stress_get_uint32(opt);
-	stress_check_range("zombie-max", zombie_max,
+	stress_check_range("zombie-max", (uint64_t)zombie_max,
 		MIN_ZOMBIES, MAX_ZOMBIES);
 	return stress_set_setting("zombie-max", TYPE_ID_INT32, &zombie_max);
 }
@@ -237,7 +237,11 @@ static int stress_zombie(const stress_args_t *args)
 				stress_zombie_head_remove(args, false);
 				continue;
 			}
-			(void)setpgid(zombie->pid, g_pgrp);
+#if defined(HAVE_GETPGID)
+			(void)setpgid(zombie->pid, getpgid(zombie->pid));
+#else
+			(void)setpgid(zombie->pid, 0);
+#endif
 
 			if (max_zombies < zombies.length)
 				max_zombies = zombies.length;
